# gui_list.py
#
# openipmi GUI handling for a list
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix
import gui_errstr
import gui

# A list widget that can be embedded in something else
class SubList(Tix.ScrolledHList):
    def __init__(self, parent, columns, options, width, height):
        Tix.ScrolledHList.__init__(self, parent,
                                   options=options, width=width, height=height)
        
        i = 0
        for c in columns:
            self.hlist.header_create(i, text=c[0])
            self.hlist.column_width(i, c[1])
            i += 1
            pass
        
        self.list_hash = { }
        self.currkey = 0

        self.hlist.bind("<Button-3>", self.ListMenu)

        self.bind("<Destroy>", self.OnDestroy)

        self.hlist.bind("<MouseWheel>", self.Wheel)
        if (gui.winsys == "x11"):
            self.hlist.bind("<Button-4>", self.ButtonUp)
            self.hlist.bind("<Button-5>", self.ButtonDown)
            pass

        return

    def Wheel(self, event):
        self.hlist.yview("scroll", -(event.delta / 20), "units")
        return
    
    def ButtonUp(self, event):
        event.delta = 120
        self.Wheel(event);
        return
    
    def ButtonDown(self, event):
        event.delta = -120
        self.Wheel(event);
        return
    
    def OnDestroy(self, event):
        self.list_hash = None
        return

    def ListMenu(self, event):
        w = event.widget
        key = w.nearest(event.y)
        data = self.list_hash[key]
        if (data and hasattr(data, "HandleMenu")):
            data.HandleMenu(event, key, event)
            pass
        return

    def DelItem(self, key):
        self.hlist.delete_entry(key)
        del self.list_hash[key]
        return

    def Append(self, name, values, data=None):
        key = str(self.currkey)
        self.currkey += 1
        self.hlist.add(key, text=name)
        self.list_hash[key] = data
        i = 1
        for v in values:
            if (v != None):
                self.hlist.item_create(key, i, text=str(v))
            i += 1
            pass
        return key

    def SetColumn(self, idx, colnum, value):
        self.hlist.item_configure(idx, colnum, text=value)
        return

    def SetColumnStyle(self, node, colnum, style):
        self.hlist.item_configure(node, colnum, style=style)
        return

    def add_data(self, name, values, data=None):
        idx = self.Append(name, values, data);
        if (data != None):
            data.SetItem(idx)
            pass
        return idx

    def DeleteAllItems(self):
        self.hlist.delete_all()
        self.list_hash = { }
        self.currkey = 0
        return
    
    pass

# A top-level list
class List(Tix.Toplevel):
    def __init__(self, name, columns):
        Tix.Toplevel.__init__(self)
        self.title(name)

        slist = SubList(self,
                        columns,
                        options=("hlist.header 1"
                                 + " hlist.itemtype text"
                                 + (" hlist.columns "
                                    + str(len(columns)))
                                 + " hlist.selectForeground black"
                                 + " hlist.selectBackground beige"),
                        width=600, height=500)
        slist.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        self.slist = slist
        
        self.errstr = gui_errstr.ErrStr(self)
        self.errstr.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        
        bbox = Tix.ButtonBox(self)
        if hasattr(self, "ok"):
            bbox.add("ok", text="Ok", command=self.ok)
            pass
        if hasattr(self, "save"):
            bbox.add("save", text="Save", command=self.save)
            pass
        if hasattr(self, "cancel"):
            bbox.add("cancel", text="Cancel", command=self.cancel)
            pass
        if hasattr(self, "clear"):
            bbox.add("clear", text="Clear", command=self.clear)
            pass
        bbox.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        self.bind("<Destroy>", self.OnDestroy)

        return

    def AfterDone(self):
        return

    def Close(self):
        self.destroy()
        return

    def OnDestroy(self, event):
        if (hasattr(self, "do_on_close")):
            self.do_on_close()
            pass
        return

    def DelItem(self, key):
        self.slist.DelItem(key)
        return

    def Append(self, name, values, data=None):
        return self.slist.Append(name, values, data)

    def SetColumn(self, idx, colnum, value):
        self.slist.SetColumn(idx, colnum, value)
        return

    def add_data(self, name, values, data=None):
        return self.slist.add_data(name, values, data)

    def DeleteAllItems(self):
        return self.slist.DeleteAllItems()
        return
    
    def SetError(self, str):
        self.errstr.SetError(str)
        return

    # Pass the rest of the functions on to the base list.
    def DelItem(self, key):
        self.slist.DelItem(key)
        return

    def Append(self, name, values, data=None):
        return self.slist.Append(name, values, data)

    def SetColumn(self, idx, colnum, value):
        self.slist.SetColumn(idx, colnum, value)
        return

    def add_data(self, name, values, data=None):
        return self.slist.add_data(name, values, data)

    def DeleteAllItems(self):
        return self.slist.DeleteAllItems()
        return
    
    pass



# _sensor.py
#
# openipmi GUI handling for sensors
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import OpenIPMI
import gui_popup
import gui_setdialog
import _oi_logging

class SensorRefreshData:
    def __init__(self, s):
        self.s = s
        return

    def sensor_cb(self, sensor):
        if (sensor.is_readable()):
            sensor.get_value(self.s)
        return

    pass


class SensorInfoGetter:
    def __init__(self, s, func):
        self.s = s;
        self.func = func;
        return

    def DoUpdate(self):
        self.s.sensor_id.to_sensor(self)
        return

    def sensor_cb(self, sensor):
        getattr(sensor, self.func)(self.s)
        return

    pass


threshold_strings = [ 'ur', 'uc', 'un', 'ln', 'lc', 'lr' ]
threshold_full_strings = [ 'upper non-recoverable',
                           'upper critical',
                           'upper non-critical',
                           'lower non-critical',
                           'lower critical',
                           'lower non-recoverable' ]

def threshold_str_to_full(s):
    i = threshold_strings.index(s)
    return threshold_full_strings[i]

threshold_event_strings = [ 'urha', 'urhd', 'urla', 'urld',
                            'ucha', 'uchd', 'ucla', 'ucld',
                            'unha', 'unhd', 'unla', 'unld',
                            'lnha', 'lnhd', 'lnla', 'lnld',
                            'lcha', 'lchd', 'lcla', 'lcld',
                            'lrha', 'lrhd', 'lrla', 'lrld' ]

def threshold_event_str_to_full(s):
    rv = threshold_str_to_full(s[0:2])
    if (s[2] == 'h'):
        rv += " going high"
    else:
        rv += " going low"
        pass
    if (s[3] == 'a'):
        rv += " assertion"
    else:
        rv += " deassertion"
        pass
    return rv

def discrete_event_str_to_full(s, name):
    l = len(s)
    num = int(s[0:l-1])
    t = s[l-1:l]
    if (t == 'a'):
        t = 'assertion'
    else:
        t = 'deassertion'
        pass
    return s[0:l-1] + ' ' + t + ' (' + name + ')'

class SensorHysteresisSet:
    def __init__(self, s):
        self.s = s
        self.setting = False
        if (s.sensor_id.to_sensor(self) != 0):
            s.gui.ReportError("Unable to convert " + s.name
                              + "to a sensor for hysteresis setting")
            pass
        return
        
    def ok(self, vals):
        self.positive = int(vals[0])
        self.negative = int(vals[1])
        rv = self.s.sensor_id.to_sensor(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting sensor thresholds: "
                    + OpenIPMI.get_error_string(rv))
        return

    def sensor_cb(self, sensor):
        if (self.setting):
            self.err = sensor.set_hysteresis(self.positive, self.negative, self)
        else:
            sensor.get_hysteresis(self)
            self.setting = True
            pass
        return

    def sensor_get_hysteresis_cb(self, sensor, err, positive, negative):
        if (err != 0):
            _oi_logging.error("Error getting sensor hysteresis: " + str(err))
            self.Destroy()
        else:
            gui_setdialog.SetDialog("Set hysteresis for " + self.s.name,
                                    [ positive, negative ], 2, self,
                                    [ "Positive", "Negative" ])
            pass
        return

    def sensor_set_hysteresis_cb(self, sensor, err):
        if (err):
            _oi_logging.error("Unable to set sensor thresholds: " + str(err))
        else:
            sensor.get_hysteresis(self.s)
            pass
        return

    pass


class SensorThresholdsSet:
    def __init__(self, s):
        self.s = s
        self.setting = False
        if (s.sensor_id.to_sensor(self) != 0):
            s.gui.ReportError("Unable to convert " + s.name
                              + "to a sensor for threshold setting")
            pass
        return

    def ok(self, vals):
        tlist = [ ]
        i = 0
        for th in self.thresholds:
            tlist.append(th + " " + str(float(vals[i])))
            i += 1
            pass
        self.threshold_str = ":".join(tlist)
        self.setting = True
        rv = self.s.sensor_id.to_sensor(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting sensor thresholds: "
                    + OpenIPMI.get_error_string(rv))
        return

    def sensor_cb(self, sensor):
        if (self.setting):
            self.err = sensor.set_thresholds(self.threshold_str, self)
        else:
            rv = sensor.get_thresholds(self)
            if (rv != 0):
                _oi_logging.error("Error getting sensor thresholds: " + str(rv))
                return
            pass
        return

    def sensor_get_thresholds_cb(self, sensor, err, th):
        if (err != 0):
            _oi_logging.error("Error getting sensor thresholds: " + str(err))
            return
        self.thresholds = [ ]
        defaults = [ ]
        labels = [ ]
        for i in th.split(':'):
            j = i.split()
            self.thresholds.append(j[0])
            defaults.append(j[1])
            labels.append(threshold_str_to_full(j[0]))
            pass
        gui_setdialog.SetDialog("Set Thresholds for " + str(self.s),
                                defaults, len(defaults), self, labels)
        return

    def sensor_set_thresholds_cb(self, sensor, err):
        if (err):
            _oi_logging.error("Unable to set sensor thresholds: " + str(err))
        else:
            sensor.get_thresholds(self.s)
            pass
        return

    pass


class SensorEventEnablesSet:
    def __init__(self, s):
        self.s = s
        self.setting = False
        if (s.sensor_id.to_sensor(self) != 0):
            s.gui.ReportError("Unable to convert " + s.name
                              + "to a sensor for event enable setting")
            pass
        return

    def ok(self, vals):
        tlist = [ ]
        if (vals[0]):
            tlist.append("events")
            pass
        del vals[0]
        if (vals[0]):
            tlist.append("scanning")
            pass
        del vals[0]

        for en in self.enables:
            if vals[0]:
                tlist.append(en)
                pass
            del vals[0]
            pass
        self.event_enable_str = " ".join(tlist)
        self.setting = True
        rv = self.s.sensor_id.to_sensor(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting sensor event enables: "
                    + OpenIPMI.get_error_string(rv))
        return
        
    def sensor_cb(self, sensor):
        if (self.setting):
            self.err = sensor.set_event_enables(self.event_enable_str, self)
        else:
            rv = sensor.get_event_enables(self)
            if (rv != 0):
                _oi_logging.error("Error getting sensor event enables: "
                                  + str(rv))
                return
            pass
        return

    def sensor_get_event_enable_cb(self, sensor, err, st):
        if (err != 0):
            _oi_logging.error("Error getting sensor event enables: "
                              + str(err))
            return
        defaults = [ False, False ]
        labels = [ "Enable Events", "Scanning" ]
        en = { }
        for i in self.s.events_supported:
            en[i] = False
            pass
        for i in st.split(' '):
            if (i == "events"):
                defaults[0] = True
            elif (i == "scanning"):
                defaults[1] = True
            elif (i == "busy"):
                pass
            elif (self.s.event_support == OpenIPMI.EVENT_SUPPORT_PER_STATE):
                en[i] = True
                pass
            pass
        
        self.enables = [ ]
        for i in self.s.events_supported:
            defaults.append(en[i])
            self.enables.append(i)
            if (self.s.is_threshold):
                labels.append(threshold_event_str_to_full(i))
            else:
                name = s.events_supported_name[i]
                labels.append(discrete_event_str_to_full(i, name))
                pass
            pass

        gui_setdialog.SetDialog("Set Event Enables for " + str(self.s),
                                defaults, len(defaults), self, labels)
        return

    def sensor_event_enable_cb(self, sensor, err):
        if (err):
            _oi_logging.error("Unable to set sensor event enables: " + str(err))
        else:
            sensor.get_event_enables(self.s)
            pass
        return

    pass


class Sensor:
    def __init__(self, e, sensor):
        if (e.ui.in_destroy):
            return
        self.e = e
        self.name = sensor.get_name()
        e.sensors[self.name] = self
        self.sensor_id = sensor.get_id()
        self.ui = e.ui
        ui = self.ui
        self.destroyed = False
        self.updater = SensorRefreshData(self)
        ui.add_sensor(self.e, self)
        self.in_warning = False
        self.in_severe = False
        self.in_critical = False
        self.ui.append_item(self, "Sensor Type",
                            sensor.get_sensor_type_string())
        self.ui.append_item(self, "Event Reading Type",
                            sensor.get_event_reading_type_string())
        m = sensor.get_mc()
        self.ui.append_item(self, "Msg Routing Info",
                            "MC: " + m.get_name()
                            + "  LUN:" + str(sensor.get_lun())
                            + "  Num:" + str(sensor.get_num()))
                            
        self.event_support = sensor.get_event_support()
        es = self.event_support
        self.ui.append_item(self, "Event Support",
                            OpenIPMI.get_event_support_string(es))
        if ((es == OpenIPMI.EVENT_SUPPORT_PER_STATE)
            or (es == OpenIPMI.EVENT_SUPPORT_ENTIRE_SENSOR)):
            self.event_enables = self.ui.append_item(self, "Event Enables",
                                      None,
                                      data = SensorInfoGetter(self,
                                                       "get_event_enables"))
            pass

        sensor.add_event_handler(self)
        if (sensor.is_readable()):
            sensor.get_value(self)
            pass
        else:
            self.ui.set_item_text(self.treeroot, "(not readable)")
            pass

        self.auto_rearm = sensor.get_supports_auto_rearm()

        self.is_threshold = (sensor.get_event_reading_type()
                             == OpenIPMI.EVENT_READING_TYPE_THRESHOLD)
        if (self.is_threshold):
            self.threshold_sensor_units = sensor.get_base_unit_string()
            modifier = sensor.get_modifier_unit_use();
            if (modifier == OpenIPMI.MODIFIER_UNIT_BASE_DIV_MOD):
               self.threshold_sensor_units += "/"
               pass
            elif (modifier == OpenIPMI.MODIFIER_UNIT_BASE_MULT_MOD):
               self.threshold_sensor_units += "*"
               pass
            modifier = sensor.get_modifier_unit_string()
            if (modifier != "unspecified"):
                self.threshold_sensor_units += modifier
                pass
            self.threshold_sensor_units += sensor.get_rate_unit_string()
            if (sensor.get_percentage()):
                self.threshold_sensor_units += '%'
                pass

            self.events_supported = [ ]
            if (es != OpenIPMI.EVENT_SUPPORT_NONE):
                for i in threshold_event_strings:
                    ival = [ 0 ]
                    rv = sensor.threshold_event_supported(i, ival)
                    if (rv == 0) and (ival[0] != 0):
                        self.events_supported.append(i)
                        pass
                    pass
                self.ui.append_item(self, "Threshold Events Supported",
                                    ' '.join(self.events_supported))

                pass
            sval = ""
            fval = [ 0.0 ]
            rv = sensor.get_nominal_reading(fval)
            if (rv == 0):
                sval += "  Nominal:" + str(fval[0])
                pass
            rv = sensor.get_sensor_min(fval)
            if (rv == 0):
                sval += "  Min:" + str(fval[0])
                pass
            rv = sensor.get_sensor_max(fval)
            if (rv == 0):
                sval += "  Max:" + str(fval[0])
                pass
            rv = sensor.get_normal_min(fval)
            if (rv == 0):
                sval += "  NormalMin:" + str(fval[0])
                pass
            rv = sensor.get_normal_max(fval)
            if (rv == 0):
                sval += "  NormalMax:" + str(fval[0])
                pass
            if (sval != ""):
                sval = sval.strip()
                self.ui.append_item(self, "Ranges", sval);
                pass

            self.threshold_support = sensor.get_threshold_access()
            ts = self.threshold_support
            self.ui.append_item(self, "Threshold Support",
                              OpenIPMI.get_threshold_access_support_string(ts))
            sval = ""
            rval = ""
            if (ts != OpenIPMI.THRESHOLD_ACCESS_SUPPORT_NONE):
                wval = ""
                ival = [ 0 ]
                for th in threshold_strings:
                    rv = sensor.threshold_settable(th, ival)
                    if (rv == 0) and (ival[0] == 1):
                        sval += " " + th
                        pass
                    rv = sensor.threshold_readable(th, ival)
                    if (rv == 0) and (ival[0] == 1):
                        rval += " " + th
                        pass
                    rv = sensor.threshold_reading_supported(th, ival)
                    if (rv == 0) and (ival[0] == 1):
                        wval += " " + th
                        pass
                    pass
                if (wval != ""):
                    wval = wval.strip()
                    self.ui.append_item(self, "Thresholds Reported", wval)
                    pass

            if ((ts == OpenIPMI.THRESHOLD_ACCESS_SUPPORT_READABLE)
                or (ts == OpenIPMI.THRESHOLD_ACCESS_SUPPORT_SETTABLE)):
                if (sval != ""):
                    sval = sval.strip()
                    self.ui.append_item(self, "Settable Thresholds", sval)
                    pass
                if (rval != ""):
                    rval = rval.strip()
                    self.ui.append_item(self, "Readable Thresholds", rval)
                    pass
                        
                self.thresholds = self.ui.append_item(self, "Thresholds",
                                      None,
                                      data = SensorInfoGetter(self,
                                                       "get_thresholds"))
                pass
            else:
                sval = ""
                pass

            self.settable_thresholds = sval

            self.hysteresis_support = sensor.get_hysteresis_support()
            hs = self.hysteresis_support
            self.ui.append_item(self, "Hysteresis Support",
                                OpenIPMI.get_hysteresis_support_string(hs))
            if ((hs == OpenIPMI.HYSTERESIS_SUPPORT_READABLE)
                or (hs == OpenIPMI.HYSTERESIS_SUPPORT_SETTABLE)):
                self.hysteresis =  self.ui.append_item(self, "Hysteresis",
                                     None,
                                     data = SensorInfoGetter(self,
                                                             "get_hysteresis"))
                pass
            pass
        else:
            self.hysteresis_support = OpenIPMI.HYSTERESIS_SUPPORT_NONE
            self.threshold_support = OpenIPMI.THRESHOLD_ACCESS_SUPPORT_NONE

            self.events_supported = [ ]
            self.events_supported_name = { }
            self.states_supported = [ ]
            self.states_supported_name = { }
            if (es != OpenIPMI.EVENT_SUPPORT_NONE):
                for i in range(0, 15):
                    ival = [ 0 ]
                    rv = sensor.discrete_event_readable(i, ival)
                    if (rv == 0) and (ival[0] != 0):
                        self.states_supported.append(str(i))
                    name = sensor.reading_name_string(i)
                    self.states_supported_name[str(i)] = name
                    for j in ['a', 'd']:
                        ival = [ 0 ]
                        sval = str(i) + j
                        rv = sensor.discrete_event_supported(sval, ival)
                        if (rv == 0) and (ival[0] != 0):
                            self.events_supported.append(sval)
                            self.events_supported_name[sval] = name
                            pass
                        pass
                    pass
                            
                self.ui.append_item(self, "Events Supported",
                                    ' '.join(self.events_supported))
                self.ui.append_item(self, "States Reported",
                                    ' '.join(self.states_supported))
                names = self.ui.append_item(self, "State Names", "")
                for i in self.states_supported:
                    self.ui.append_item(None, str(i),
                                        self.states_supported_name[str(i)],
                                        parent=names)
                    pass
                pass
            pass
        return

    def __str__(self):
        return self.name

    def DoUpdate(self):
        self.sensor_id.to_sensor(self.updater)
        return

    def HandleMenu(self, event):
        if (self.impt_data == None):
            l = [ [ "Add to watch values", self.add_impt ] ]
            pass
        else:
            l = [ [ "Remove from watch values", self.remove_impt ] ]
            pass
        if (self.event_support != OpenIPMI.EVENT_SUPPORT_NONE):
            l.append( ("Rearm", self.Rearm) )
            pass
        if (self.threshold_support == OpenIPMI.THRESHOLD_ACCESS_SUPPORT_SETTABLE):
            l.append( ("Set Thresholds", self.SetThresholds) )
            pass
        if (self.hysteresis_support == OpenIPMI.HYSTERESIS_SUPPORT_SETTABLE):
            l.append( ("Set Hysteresis", self.SetHysteresis) )
            pass
        if ((self.event_support == OpenIPMI.EVENT_SUPPORT_PER_STATE)
            or (self.event_support == OpenIPMI.EVENT_SUPPORT_ENTIRE_SENSOR)):
            l.append( ("Set Event Enables", self.SetEventEnables) )
            pass

        if (len(l) > 0):
            gui_popup.popup(self.ui, event, l)
            pass
        return

    def add_impt(self, event):
        self.ui.add_impt_data("sensor", str(self), self)
        return
        
    def remove_impt(self, event):
        self.ui.remove_impt_data(self.impt_data)
        return
        
    def Rearm(self, event):
        return
    
    def SetThresholds(self, event):
        SensorThresholdsSet(self)
        return
    
    def SetHysteresis(self, event):
        SensorHysteresisSet(self)
        return
    
    def SetEventEnables(self, event):
        SensorEventEnablesSet(self)
        return
    
    def remove(self):
        self.e.sensors.pop(self.name)
        self.ui.remove_sensor(self)
        self.destroyed = True
        self.e = None
        self.updater = None
        self.ui = None
        return

    def handle_threshold_states(self, states):
        if (self.destroyed):
            return
        th = states.split()
        while len(th) > 0:
            v = th[0]
            del th[0]

            if (v == "un") or (v == "ln"):
                if (not self.in_warning):
                    self.in_warning = True
                    self.ui.incr_item_warning(self.treeroot)
                    pass
                pass
            else:
                if (self.in_warning):
                    self.in_warning = False
                    self.ui.decr_item_warning(self.treeroot)
                    pass
                pass

            if (v == "uc") or (v == "lc"):
                if (not self.in_severe):
                    self.in_severe = True
                    self.ui.incr_item_severe(self.treeroot)
                    pass
                pass
            else:
                if (self.in_severe):
                    self.in_severe = False
                    self.ui.decr_item_severe(self.treeroot)
                    pass
                pass

            if (v == "ur") or (v == "lr"):
                if (not self.in_critical):
                    self.in_critical = True
                    self.ui.incr_item_critical(self.treeroot)
                    pass
                pass
            else:
                if (self.in_critical):
                    self.in_critical = False
                    self.ui.decr_item_critical(self.treeroot)
                    pass
                pass
            pass
        return

    def threshold_reading_cb(self, sensor, err, raw_set, raw, value_set,
                             value, states):
        if (self.destroyed):
            return
        if (err):
            self.ui.set_item_text(self.treeroot, None)
            return
        v = ""
        if (value_set):
            v += str(value) + self.threshold_sensor_units
            pass
        if (raw_set):
            v += " (" + str(raw) + ")"
            pass
        v += ": " + states
        self.ui.set_item_text(self.treeroot, v)
        self.handle_threshold_states(states)
        return
        
    def discrete_states_cb(self, sensor, err, states):
        if (self.destroyed):
            return
        if (err):
            self.ui.set_item_text(self.treeroot, None)
            return
        self.ui.set_item_text(self.treeroot, states)
        return
        
    def sensor_get_event_enable_cb(self, sensor, err, states):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.event_enables, None)
            return
        self.ui.set_item_text(self.event_enables, states)
        return

    def sensor_get_hysteresis_cb(self, sensor, err, positive, negative):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.hysteresis, None)
            return
        self.ui.set_item_text(self.hysteresis,
                              "Positive:" + str(positive)
                              + " Negative:" + str(negative))
        return

    def sensor_get_thresholds_cb(self, sensor, err, th):
        if (self.destroyed):
            return
        if (err != 0):
            self.ui.set_item_text(self.thresholds, None)
            return
        self.ui.set_item_text(self.thresholds, th)
        return

    def threshold_event_cb(self, sensor, event_spec, raw_set, raw,
                           value_set, value, event):
        if (self.destroyed):
            return OpenIPMI.EVENT_NOT_HANDLED
        self.handle_threshold_states(event_spec)
        sensor.get_value(self)
        return OpenIPMI.EVENT_NOT_HANDLED
        
    def discrete_event_cb(self, sensor, event_spec, severity, old_severity,
                          event):
        if (self.destroyed):
            return OpenIPMI.EVENT_NOT_HANDLED
        return OpenIPMI.EVENT_NOT_HANDLED
    
    pass



# _conn.py
#
# openipmi GUI handling for connections
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import gui_popup
import gui_SoL

class Port:
    def __init__(self, domain, c, pnum):
        self.c = c
        self.pnum = pnum
        self.ui = c.ui
        self.name = c.d.name + "(" + str(c.cnum) + "." + str(pnum) + ")"

        c.ports[pnum] = self

        self.ui.add_port(c, self)

        # Wait to get activation information
        self.up = False
        self.ui.incr_item_warning(self.treeroot)

        pi = domain.get_port_info(c.cnum, pnum);
        if (pi != None):
            self.ui.set_item_text(self.treeroot, pi)
            pass

        v = [ 0 ]
        rv = domain.is_connection_port_up(c.cnum, pnum, v)
        if (rv == 0):
            self.SetUp(domain, v[0])
        return

    def remove(self):
        self.c = None
        self.ui = None
        return

    def __str__(self):
        return self.name

    def SetUp(self, domain, up):
        pi = domain.get_port_info(self.c.cnum, self.pnum);
        if (pi != None):
            self.ui.set_item_text(self.treeroot, pi)
            pass

        if (up):
            if (not self.up):
                self.ui.decr_item_warning(self.treeroot)
                pass
            pass
        else:
            if (self.up):
                self.ui.incr_item_warning(self.treeroot)
                pass
            pass
        self.up = up
        return

    def IsUp(self):
        return self.up

    pass

class Connection:
    def __init__(self, domain, d, cnum):
        self.d = d
        self.cnum = cnum
        self.ui = d.ui
        self.name = d.name + "(" + str(cnum) + ")"
        self.ports = { }
        self.domain_id = domain.get_id()

        d.connections[cnum] = self

        v = [ 0 ]
        rv = domain.is_connection_active(cnum, v)
        if (rv == 0):
            if (v[0] != 0):
                self.active_str = "active"
                pass
            else:
                self.active_str = "inactive"
                pass
            pass
        else:
            self.active_str = "inactive(" + str(rv) + ")"
            pass

        self.ui.add_connection(d, self)
        self.ui.set_item_text(self.treeroot,
                              (domain.get_connection_type(cnum)
                               + " (" + self.active_str + ")"))

        v = [ 0 ]
        rv = domain.num_connection_ports(cnum, v)
        if (rv == 0):
            num_ports = v[0]
            for i in range(0, num_ports):
                Port(domain, self, i)
                pass
            pass
        self.conup = False
        self.ui.incr_item_severe(self.treeroot)
        return

    def remove(self):
        self.ui = None
        for p in self.ports.itervalues():
            p.remove()
            pass
        self.ports = None
        return
    
    def __str__(self):
        return self.name

    def DoUpdate(self):
        self.op = "checkactive"
        self.domain_id.to_domain(self)
        return
    
    def SetPortUp(self, domain, port, err):
        if (port not in self.ports):
            Port(domain, self, port)
            pass
        if (err == OpenIPMI.enoent):
            self.ui.remove_port(self.ports[port])
            del self.ports[port];
            return
        
        self.ports[port].SetUp(domain, err == 0)
        conup = False
        for p in self.ports.itervalues():
            conup = p.IsUp() or conup
            pass
        if (conup):
            if (not self.conup):
                self.ui.decr_item_severe(self.treeroot)
                pass
            pass
        else:
            if (self.conup):
                self.ui.incr_item_severe(self.treeroot)
                pass
            pass
        self.conup = conup
        return

    def IsPortUp(self, port):
        return self.ports[port].IsUp(port)

    def IsUp(self):
        return self.conup
    
    def HandleMenu(self, event):
        gui_popup.popup(self.ui, event,
                        [ [ "Activate", self.Activate ],
                          [ "Open SOL", self.OpenSOL ] ])
        return

    def Activate(self, event):
        self.op = "activate"
        self.domain_id.to_domain(self)
        return

    def OpenSOL(self, event):
        gui_SoL.SoL(self.ui, self.domain_id, self.cnum)
        return
    
    def domain_cb(self, domain):
        if (self.op == "activate"):
            domain.activate_connection(self.cnum)
        elif (self.op == "checkactive"):
            v = [ 0 ]
            rv = domain.is_connection_active(self.cnum, v)
            if (rv == 0):
                if (v[0] != 0):
                    new_active_str = "active"
                    pass
                else:
                    new_active_str = "inactive"
                    pass
                pass
            else:
                new_active_str = "inactive("+ str(rv) + ")"
                pass

            if (new_active_str != self.active_str):
                self.active_str = new_active_str
                self.ui.set_item_text(self.treeroot,
                                      (domain.get_connection_type(self.cnum)
                                       + " (" + self.active_str + ")"))
                pass
            pass
        return

    pass



# gui_lightset.py
#
# openipmi GUI handling for light setting
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix

class EnumHolder:
    def __init__(self):
        self.value = ""
        return

    def get(self):
        return self.value

    def set(self, val):
        self.value = val

    def SelectType(self, newbutton, selected):
        if (selected == "0"):
            return
        self.value = newbutton
        return
    
    pass

class LightSet(Tix.Toplevel):
    def __init__(self, name, num_vals, light_list, vals, handler):
        self.handler = handler
        self.light_list = light_list
        Tix.Toplevel.__init__(self)
        self.title(name)

        svalues = Tix.ScrolledWindow(self)
        self.values = svalues.window
        self.lights = [ ]

        row = 0
        for i in range(0, num_vals):
            if (len(vals) <= i):
                ivals = ("", "black", '0', '1')
            else:
                ivals = vals[i]
                pass

            label = Tix.Label(self.values, text="Light " + str(i))
            label.grid(row=row, column=0, sticky="e")
            if (light_list[i][0]):
                lc = Tix.BooleanVar()
                lcw = Tix.Checkbutton(self.values, text="Local Control",
                                      variable=lc)
                lc.set(ivals[0] == "lc")
                lcw.grid(row=row, column=1, sticky="ew")
                row += 1
            else:
                lc = None
                pass

            color = EnumHolder()
            colorw = Tix.Select(self.values, label="Color", allowzero=0,
                               radio=1, command=color.SelectType)
            for v in light_list[i][1]:
                colorw.add(v, text=v)
                pass
            colorw.configure(value=ivals[1])
            colorw.grid(row=row, column=1, sticky="ew")
            row += 1

            ontime = Tix.LabelEntry(self.values, label="On Time")
            ontime.entry.insert("0", ivals[2])
            ontime.grid(row=row, column=1, sticky="ew")
            row += 1
            ontime = ontime.entry

            offtime = Tix.LabelEntry(self.values, label="Off Time")
            offtime.entry.insert("0", ivals[3])
            offtime.grid(row=row, column=1, sticky="ew")
            row += 1
            offtime = offtime.entry

            self.lights.append( (lc, color, ontime, offtime) )
            pass
            
        svalues.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)

        bbox = Tix.ButtonBox(self)
        bbox.add("cancel", text="Cancel", command=lambda w=self: w.cancel())
        bbox.add("ok", text="Ok", command=lambda w=self: w.ok_press())
        bbox.pack(side=Tix.BOTTOM, fill=Tix.X, expand=1)

        self.bind("<Destroy>", self.OnDestroy)
        return

    def cancel(self):
        self.destroy()
        return

    def ok_press(self):
        val = [ ]
        try:
            i = 0;
            for f in self.lights:
                lc = ""
                if (f[0] != None) and f[0].get():
                    lc = str("lc")
                color = str(f[1].get())
                ontime = str(f[2].get())
                offtime = str(f[3].get())
                val.append(' '.join([lc, color, ontime, offtime]))
                i = i + 1
                pass

            self.handler.ok(val)
            pass
        except Exception, e:
            print str(e)
            return
        self.destroy()
        return

    def OnDestroy(self, event):
        if (hasattr(self, "do_on_close")):
            self.do_on_close()
            pass
        self.handler = None
        self.light_list = None
        self.lights = None
        return

    pass



# _term.py
#
# openipmi generic terminal handling for SoL
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

black = 0
red = 1
green = 2
yellow = 3
blue = 4
magenta = 5
cyan = 6
white = 7

BOLD = 1
UNDERLINE = 2
INVERSE = 4
BLINK = 8
CONCEALED = 16

class TerminalEmulator:
    def __init__(self):
        self.buf = [ ]
        # Mode is a 2-d array of [ cflags, bg color, fg color ]
        self.modes = [ ]
        # Style is (fg, bg, flags)
        for i in range(0, 24):
            self.buf.append([ ])
            self.modes.append([ ])
            for j in range(0, 80):
                self.buf[i].append(" ")
                self.modes[i].append( [0, 0, 7] )
                pass
            pass
        self.cflags = 0
        self.bg_color = black
        self.fg_color = white
        self.x = 0
        self.y = 0
        self.height = 24
        self.width = 80
        self.InputHandler = self.Input0
        self.parms = None
        self.saved_pos = [1, 1]
        self.scroll_region = [0, self.height-1]
        self.keypad_alt = False
        return

    def check_scroll_down(self):
        redraw = False
        starty = self.GetStartY()
        endy = self.GetEndY()
        if (self.y == endy):
            old = self.buf[starty]
            del(self.buf[starty])
            self.buf.insert(endy, old)
            old = self.modes[starty]
            del(self.modes[starty])
            self.modes.insert(endy, old)
            for j in range(0, self.width):
                self.buf[endy][j] = " "
                self.modes[endy][j][0] = 0
                self.modes[endy][j][1] = black
                self.modes[endy][j][2] = white
                pass
            pass
            self.ScrollLines(starty, endy)
        else:
            self.y += 1
            pass
        return

    def check_scroll_up(self):
        starty = self.GetStartY()
        endy = self.GetEndY()
        if (self.y == starty):
            old = self.buf[endy]
            del(self.buf[endy])
            self.buf.insert(starty, old)
            for j in range(0, self.width):
                self.buf[starty][j] = ' '
                self.modes[starty][j][0] = 0
                self.modes[starty][j][1] = black
                self.modes[starty][j][2] = white
                pass
            pass
            self.ScrollLinesUp(starty, endy)
        else:
            self.y -= 1
            pass
        return

    def output_at(self, x, y, slen):
        s = ""
        i = 0
        last_mode = [ 0, 0, 7 ]
        lastx = 0
        while (i < slen):
            if (len(s) > 0):
                if (last_mode != self.modes[y][x+i]):
                    # Character mode change, output what we have and switch.
                    self.DrawText(last_mode[2], last_mode[1], last_mode[0],
                                  x, y, s)
                    s = ""
                    pass
                pass
            else:
                last_mode = self.modes[y][x+i]
                lastx = x + i
                pass
            s += self.buf[y][x+i]
            i += 1
            pass
        self.DrawText(last_mode[2], last_mode[1], last_mode[0],
                      lastx, y, s)
        return

    def handle_cursor(self):
        if (self.x == self.width):
            xpos = self.x - 1
        else:
            xpos = self.x
            pass
        mode = self.modes[self.y][xpos]
        self.DrawCursor(mode[2], mode[1], mode[0], xpos, self.y,
                        self.buf[self.y][xpos])
        return

    def restore_cursor(self):
        if (self.x == self.width):
            self.output_at(self.x-1, self.y, 1)
            pass
        else:
            self.output_at(self.x, self.y, 1)
            pass
        return
        
    def output_str(self, s):
        if (len(s) == 0):
            return
        do_scroll = False
        if (self.x == self.width):
            # Cursor is at the end of the line, redraw the char where
            # the cursor is sitting.
            self.restore_cursor()
            self.x = 0;
            self.check_scroll_down()
            pass
        while ((self.x + len(s)) > self.width):
            # The string exceeds the line length, do it in pieces
            outlen = self.width - self.x
            do_scroll = True
            self.buf[self.y][self.x:self.x+outlen] = s[0:outlen]
            self.output_at(self.x, self.y, outlen)
            self.x = 0;
            s = s[outlen:]
            self.check_scroll_down()
            pass
        outlen = len(s)
        for i in range(0, outlen):
            self.buf[self.y][self.x+i] = s[i]
        for i in range(self.x, self.x+outlen):
            self.modes[self.y][i][0] = self.cflags
            self.modes[self.y][i][1] = self.bg_color
            self.modes[self.y][i][2] = self.fg_color
            pass
        self.output_at(self.x, self.y, outlen)
        self.x += outlen
        self.handle_cursor()
        return

    def GetParm(self, n, default = 1):
        if (self.parms == None):
            return default
        elif (len(self.parms) <= n):
            return default
        return self.parms[n]

    # Get the current upper bound of the cursor, used for scroll regions.
    def GetStartY(self):
        if ((self.y >= self.scroll_region[0])
            and (self.y <= self.scroll_region[1])):
            return self.scroll_region[0]
        return 0

    # Get the current lower bound of the cursor, used for scroll regions.
    def GetEndY(self):
        if ((self.y >= self.scroll_region[0])
            and (self.y <= self.scroll_region[1])):
            return self.scroll_region[1]
        return self.height - 1

    # After '\e['
    def Input2(self, c, s):
        if (c == 'A'): # Up
            count = self.GetParm(0)
            starty = self.GetStartY()
            self.restore_cursor()
            if (count > (self.y - starty)):
                self.y = starty
            else:
                self.y -= count
                pass
            self.handle_cursor();
            pass
        elif (c == 'B'): # Down
            count = self.GetParm(0)
            endy = self.GetEndY()
            self.restore_cursor()
            if (count > (endy - self.y)):
                self.y = endy
            else:
                self.y += count
                pass
            self.handle_cursor();
            pass
        elif (c == 'C'): # Right
            count = self.GetParm(0)
            self.restore_cursor()
            if (count > (self.width - self.x - 1)):
                self.x = self.width - 1
            else:
                self.x += count
                pass
            self.handle_cursor();
            pass
        elif (c == 'D'): # Right
            count = self.GetParm(0)
            self.restore_cursor()
            if (count > self.x):
                self.x = 0
            else:
                self.x -= count
                pass
            self.handle_cursor();
            pass
        elif (c == '?'): # FIXME: Not sure what this does
            return "" # Stay in Input2
        elif ((c >= '0') and (c <= '9')):
            if (self.parms == None):
                self.parms = [ int(c) ]
            else:
                currparm = len(self.parms) - 1
                self.parms[currparm] *= 10
                self.parms[currparm] += int(c)
                pass
            return "" # Stay in Input2
        elif (c == ';'): # Next parm
            if (self.parms == None):
                self.parms = [ 0, 0 ]
            else:
                self.parms.append(0)
                pass
            return "" # Stay in Input2
        elif (c == 'r'): # Scroll region
            y1 = self.GetParm(0, -1)
            y2 = self.GetParm(1, -1)
            if ((y1 == -1) or (y2 == -1)):
                if ((y1 == -1) and (y2 == -1)):
                    self.scroll_region[0] = 0
                    self.scroll_region[1] = self.height - 1
                    pass
                pass
            elif ((y1 > y2) or (y1 < 1) or (y2 < 1)
                  or (y1 >= (self.height+1)) or (y2 >= (self.height+1))):
                # Bogus values, just ignore them.
                pass
            else:
                self.scroll_region[0] = y1 - 1
                self.scroll_region[1] = y2 - 1
                pass
            pass
        elif ((c == 'H') or (c == 'f')): # Move to position specified
            y = self.GetParm(0)
            x = self.GetParm(1)
            if (x < 1):
                x = 1
                pass
            elif (x > self.width):
                x = self.width
                pass
            if (y < 1):
                y = 1
                pass
            if (y > self.height):
                y = self.height
                pass
            self.restore_cursor()
            self.x = x - 1
            self.y = y - 1
            self.handle_cursor();
            pass
        elif (c == 's'): # save cursor position
            self.saved_pos[0] = self.x
            self.saved_pos[1] = self.y
            pass
        elif (c == 'u'): # Restore cursor position
            self.restore_cursor()
            self.x = self.saved_pos[0]
            self.y = self.saved_pos[1]
            self.handle_cursor();
            pass
        elif (c == 'J'): # Clear screen area
            mode = self.GetParm(0, -1)
            if (mode == -1):
                starty = self.y
                length = self.height - self.y
                pass
            elif (mode == 1):
                starty = 0
                length = self.y + 1
                pass
            elif (mode == 2):
                starty = 0
                length = self.height
                pass
            else:
                starty = 0
                length = 0
                pass
            for y in range(starty, starty + length):
                for x in range(0, self.width):
                    self.buf[y][x] = " "
                    self.modes[y][x][0] = 0
                    self.modes[y][x][1] = black
                    self.modes[y][x][2] = white
                    pass
                self.output_at(0, y, self.width)
                pass
            self.handle_cursor();
            pass
        elif (c == 'K'): # Clear line
            mode = self.GetParm(0, 0)
            y = self.y
            if (mode == 0): # To end of line
                startx = self.x
                length = self.width - self.x
                pass
            elif (mode == 1): # To start of line
                startx = 0
                length = self.x + 1
                pass
            elif (mode == 2): # Whole line
                startx = 0
                length = self.width
                pass
            else: # Ignore
                startx = 0
                length = 0
                pass
            for x in range(startx, startx+length):
                self.buf[y][x] = " "
                self.modes[y][x][0] = 0
                self.modes[y][x][1] = black
                self.modes[y][x][2] = white
                pass
            self.output_at(startx, y, length)
            self.handle_cursor();
        elif (c == 'm'): # Graphics mode
            i = 0
            val = self.GetParm(i, 0)
            while (val != -1):
                if (val == 0):
                    self.cflags = 0
                    self.bg_color = black
                    self.fg_color = white
                elif (val == 1):
                    self.cflags |= BOLD
                elif (val == 4):
                    self.cflags |= UNDERLINE
                elif (val == 5):
                    self.cflags |= BLINK
                elif (val == 7):
                    self.cflags |= INVERSE
                elif (val == 8):
                    self.cflags |= CONCEALED
                elif ((val >= 30) and (val <= 37)):
                    self.fg_color = val - 30
                elif ((val >= 40) and (val <= 47)):
                    self.bg_color = val - 40
                    pass
                i += 1
                val = self.GetParm(i, -1)
                pass
            pass
        elif (c == 'g'):
             # FIXME: \e[2g means clear tabs, so does 3g, 0g or just g
             # means current tab
            pass
        elif (c == 'P'): # delete parm characters (1 default)
            count = self.GetParm(0, 1)
            if (count > (self.width - self.x)):
                count = self.width - self.x
                pass
            y = self.y
            x = self.x
            for i in range(0, count):
                del self.buf[y][x]
                self.buf[y].append(' ')
                old = self.modes[y][x]
                del self.modes[y][x]
                old[0] = 0
                old[1] = black
                old[2] = white
                self.modes[y].append(old)
                pass
            self.DeleteChars(x, y, count)
            self.handle_cursor()
            pass
        elif (c == 'M'): # delete parm lines (1 default)
            count = self.GetParm(0, 1)
            if (count > (self.height - self.y)):
                count = self.height - self.y
                pass
            for i in range(0, count):
                old = self.buf[self.y]
                del self.buf[self.y]
                self.buf.append(old)
                old = self.modes[self.y]
                del self.modes[self.y]
                self.modes.append(old)
                for j in range(0, self.width):
                    self.buf[self.height-1][j] = ' '
                    self.modes[self.height-1][j][0] = 0
                    self.modes[self.height-1][j][1] = black
                    self.modes[self.height-1][j][2] = white
                    pass
                self.ScrollLines(self.y, self.height-1)
                pass
            self.handle_cursor()
            pass
        elif (c == 'L'): # insert parm lines (1 default)
            self.restore_cursor()
            count = self.GetParm(0, 1)
            if (count > (self.height - self.y)):
                count = self.height - self.y
                pass
            for i in range(0, count):
                old = self.buf[self.height-1]
                del self.buf[self.height-1]
                self.buf.insert(self.y, old)
                for j in range(0, self.width):
                    self.buf[self.y][j] = ' '
                    self.modes[self.y][j][0] = 0
                    self.modes[self.y][j][1] = black
                    self.modes[self.y][j][2] = white
                    pass
                self.ScrollLinesUp(self.y, self.height-1)
                pass
            self.handle_cursor()
            pass
        elif (c == 'Z'): # Move to previous tab stop
            self.restore_cursor()
            self.x = ((self.x-1) / 8) * 8
            self.handle_cursor();
            pass
        elif (c == '@'): # insert parm chars (1 default)
            self.restore_cursor()
            count = self.GetParm(0, 1)
            if (count > (self.width - self.x)):
                count = self.width - self.x
                pass
            y = self.y
            x = self.x
            for i in range(0, count):
                del self.buf[y][self.width-1]
                old = self.modes[y][self.width-1]
                del self.modes[y][self.width-1]
                old[0] = 0
                old[1] = black
                old[2] = white
                self.buf[y].insert(self.x, ' ')
                self.modes[y].insert(self.x, old)
                pass
            self.InsertChars(x, y, count)
            self.handle_cursor()
            pass
        elif (c == 'S'): # scroll forward parm lines (1 default)
            self.restore_cursor()
            count = self.GetParm(0, 1)
            if (count > self.height):
                count = self.height
                pass
            for i in range(0, count):
                old = self.buf[0]
                del self.buf[0]
                self.buf.append(old)
                old = self.modes[0]
                del self.modes[0]
                self.modes.append(old)
                for j in range(0, self.width):
                    self.buf[self.height-1][j] = ' '
                    self.modes[self.height-1][j][0] = 0
                    self.modes[self.height-1][j][1] = black
                    self.modes[self.height-1][j][2] = white
                    pass
                self.ScrollLines(0, self.height-1)
                pass
            self.handle_cursor()
            pass
        elif (c == 'T'): # scroll back parm lines (1 default)
            self.restore_cursor()
            count = self.GetParm(0, 1)
            if (count > self.height):
                count = self.height
                pass
            for i in range(0, count):
                old = self.buf[self.height-1]
                del self.buf[self.height-1]
                self.buf.insert(0, old)
                old = self.modes[self.height-1]
                del self.modes[self.height-1]
                self.modes.insert(0, old)
                for j in range(0, self.width):
                    self.buf[0][j] = ' '
                    self.modes[0][j][0] = 0
                    self.modes[0][j][1] = black
                    self.modes[0][j][2] = white
                    pass
                self.ScrollLinesUp(0, self.height-1)
                pass
            self.handle_cursor()
            pass
        elif (c == 'G'): # Move cursor to column parm (1 default)
            self.restore_cursor()
            pos = self.GetParm(0, 1)
            if (pos > self.width):
                pos = self.width
                pass
            elif (pos < 1):
                return ""
            self.x = pos - 1
            self.handle_cursor()
            pass
        elif (c == 'd'): # Move cursor to line parm (1 default)
            self.restore_cursor()
            pos = self.GetParm(0, 1)
            if (pos > self.height):
                pos = self.height
                pass
            elif (pos < 1):
                return ""
            self.y = pos - 1
            self.handle_cursor()
            pass
        elif (c == 'X'): # erase parm characters (1 default) (no cursor move)
            count = self.GetParm(0, 1)
            if (count > (self.height - self.y)):
                count = self.height - self.y
                pass
            for i in range(0, count):
                self.buf[self.y][self.x+i] = " "
                self.modes[self.y][self.x+i][0] = 0
                self.modes[self.y][self.x+i][1] = black
                self.modes[self.y][self.x+i][2] = white
                pass
            self.output_at(self.x, self.y, count)
            self.handle_cursor()
            pass
        elif (c == 'c'): # Identify terminal
            # Identify ourself as "linux"
            self.HandleTerminalOutput("\x1b[?62;9;c")
            pass
        self.InputHandler = self.Input0
        return ""

    # After an escape
    def Input1(self, c, s):
        if (c == '['):
            self.InputHandler = self.Input2
            return ""
        elif (c == 'D'): # Scroll down
            self.restore_cursor()
            self.check_scroll_down()
            self.handle_cursor();
        elif (c == 'M'): # Scroll up
            self.restore_cursor()
            self.check_scroll_up()
            self.handle_cursor();
            pass
        elif (c == 'H'): # FIXME: Set tabulator stop in all rows at current column
            pass
        elif (c == 'c'): # reset terminal
            for y in range(0, self.height):
                for x in range(0, self.width):
                    self.buf[y][x] = " "
                    self.modes[y][x][0] = 0
                    self.modes[y][x][1] = black
                    self.modes[y][x][2] = white
                    pass
                self.output_at(0, y, self.width)
                pass
            self.x = 0
            self.y = 0
            self.cflags = 0
            self.bg_color = black
            self.fg_color = white
            self.handle_cursor();
            pass
        elif ((c >= '0') and (c <= '9')):
            if (self.parms == None):
                self.parms = [ int(c) ]
            else:
                currparm = len(self.parms) - 1
                self.parms[currparm] *= 10
                self.parms[currparm] += int(c)
                pass
            return "" # Stay in Input1
        elif (c == 'n'): # Terminal state
            op = self.GetParm(0, 0)
            if (op == 5): # Requesting terminal status
                self.HandleTerminalOutput("\x1b0n") # We are ok
            elif (op == 6): # Current cursor position
                self.HandleTerminalOutput("\x1b%d;%dR" % (self.y+1, self.x+1))
            pass
        elif (c == '='): # alternate keypad mode
            self.keypat_alt = True
            pass
        elif (c == '>'): # alternate keypad mode off
            self.keypat_alt = False
            pass
        self.InputHandler = self.Input0
        return ""

    # "normal" input mode
    def Input0(self, c, s):
        if ((c >= ' ') and (c <= '~')):
            return s + c
        else:
            self.output_str(s)
            if (c == '\n'):
                self.restore_cursor()
                self.check_scroll_down()
                self.handle_cursor();
                pass
            elif (c == '\r'):
                self.restore_cursor()
                self.x = 0
                self.handle_cursor();
                pass
            elif (c == '\t'):
                self.restore_cursor()
                if (self.x >= self.width-8):
                    self.x = self.width-1
                else:
                    self.x = ((self.x / 8) * 8) + 8
                    pass
                self.handle_cursor();
                pass
            elif (c == '\007'): #bell
                self.Bell()
            elif (c == '\010'): #backspace
                if (self.x > 0):
                    self.restore_cursor()
                    self.x -= 1
                    self.handle_cursor();
                    pass
                pass
            elif (c == '\x1b'):
                self.parms = None
                self.InputHandler = self.Input1
                pass
            pass
        return ""
    
    def ProcessInput(self, data):
        s = ""
        for c in data:
            s = self.InputHandler(c, s)
            pass
        self.output_str(s)
        return

    def ResizeTerminal(self, w, h):
        return

    def Width(self):
        return self.width

    def Height(self):
        return self.height

    def Update(self):
        return

    def Reset(self):
        return
    
    pass





# _domainDialog.py
#
# Prompts for creating a domain in the openipmi GUI
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import Tix
import OpenIPMI
import _domain
import gui_errstr

class EnumHolder:
    def __init__(self):
        self.value = ""
        return

    def get(self):
        return self.value

    def set(self, val):
        self.value = val

    def SelectType(self, newbutton, selected):
        if (selected == "0"):
            return
        self.value = newbutton
        return
    
    pass

class ConnTypeInfo(Tix.Frame):
    def __init__(self, contype, parent):
        Tix.Frame.__init__(self, parent)
        self.contype = contype
        self.fields = [ ]

        args = OpenIPMI.alloc_empty_args(str(contype))
        self.args = args

        self.errstr = gui_errstr.ErrStr(self)
        self.errstr.pack(side=Tix.TOP, fill=Tix.X, expand=1)

        i = 0
        rv = 0
        frame = None
        while (rv == 0):
            name = [ "" ]
            vtype = [ "" ]
            vrange = [ "" ]
            vhelp = [ "" ]
            value = [ "" ]
            rv = args.get_val(i, name, vtype, vhelp, value, vrange)
            if (rv == 0):
                if (vhelp[0][0] == '*'):
                    optional = False
                    vhelp[0] = vhelp[0][1:]
                else:
                    optional = True
                    pass
                if (vhelp[0][0] == '!'):
                    password = True
                    vhelp[0] = vhelp[0][1:]
                else:
                    password = False
                    pass
                
                if (vtype[0] == "bool"):
                    if (frame == None):
                        didframe = True
                        frame = Tix.Frame(self)
                        frame.pack(side=Tix.TOP)
                        newframe = frame
                        pass
                    else:
                        newframe = None
                        pass
                    fw = Tix.BooleanVar()
                    w = Tix.Checkbutton(frame, text=name[0], variable=fw)
                    w.pack(side=Tix.LEFT, padx=10)
                    if ((value[0] != None) and (value[0] == "true")):
                        fw.set(True)
                        pass
                    pass
                elif (vtype[0] == "enum"):
                    do_box = False
                    fw = EnumHolder()
                    if (value[0] != None):
                        fw.set(value[0])
                        pass
                    else:
                        fw.set(vrange[0])
                        pass
                    w = Tix.Select(self, label=name[0], allowzero=0, radio=1,
                                   command=fw.SelectType)
                    for v in vrange:
                        w.add(v, text=v)
                        pass
                    newframe = None
                    w.invoke(fw.get())
                    w.pack()
                    pass
                elif (vtype[0] == "str") or (vtype[0] == "int"):
                    if (frame == None):
                        didframe = True
                        frame = Tix.Frame(self)
                        frame.pack(side=Tix.TOP)
                        newframe = frame
                        pass
                    else:
                        newframe = None
                        pass

                    if (value[0] == None):
                        value[0] = ""
                        pass
                    if (password):
                        options="entry.show '*' entry.width 20"
                    else:
                        options="entry.width 20"
                        pass
                    w = Tix.LabelEntry(frame, label=name[0], options=options)
                    w.entry.insert(Tix.END, value[0])
                    w.pack(side=Tix.LEFT, padx=10)
                    fw = w.entry
                    pass
                else:
                    i += 1
                    continue
                frame = newframe
                self.fields.append( (i, name[0], vtype[0], fw) )
                pass
            i += 1
            pass

        return

    def SetupArgs(self):
        self.errstr.SetError("")
        args = self.args
        for f in self.fields:
            idx = f[0]
            vtype = f[2]
            fw = f[3]
            if (vtype == "bool"):
                if (fw.get()):
                    val = "true"
                else:
                    val = "false"
                    pass
                pass
            elif (vtype == "enum"):
                val = str(fw.get())
                pass
            elif ((vtype == "str") or (vtype == "int")):
                val = str(fw.get())
                if (val == ""):
                    val = None
                    pass
                pass
            else:
                continue
            rv = args.set_val(idx, None, val);
            if (rv != 0):
                err = ("Error setting field " + f[1] + ": "
                       + OpenIPMI.get_error_string(rv))
                self.errstr.SetError(err)
                raise Exception(err)
            pass
        return args

    def Cleanup(self):
        self.args = None
        return

    pass

class ConnInfo(Tix.ScrolledWindow):
    def __init__(self, parent, mainhandler, enable=True):
        self.contypes = { }
        OpenIPMI.parse_args_iter_help(self)
        if (len(self.contypes) == 0):
            return
        
        Tix.ScrolledWindow.__init__(self, parent, height=300, width=600)
        self.parent = parent
        self.enable = enable;

        if (not enable):
            self.enablevar = Tix.BooleanVar()
            self.enable_box = Tix.Checkbutton(self.window, text="Enable",
                                              variable=self.enablevar)
            self.enable_box.pack(side=Tix.TOP)
            pass

        self.contype = Tix.Select(self.window, label="Connection Type",
                                  radio=1, allowzero=0,
                                  command=self.selectType)
        for key in self.contypes.keys():
            self.contype.add(key, text=key)
            pass
        self.contype.pack(side=Tix.TOP)

        self.coninfos = [ ]

        show = True
        panel = Tix.Frame(self.window)
        panel.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        for ct in self.contypes.keys():
            cti = ConnTypeInfo(ct, panel)
            if (show):
                cti.pack()
                self.currcon = cti
                show = False
                pass
            self.coninfos.append(cti)
            pass

        self.contype.invoke(self.contypes.keys()[0])

        return

    def parse_args_iter_help_cb(self, name, help):
        self.contypes[name] = help
        return

    def selectType(self, newbutton, selected):
        if (selected == "0"):
            return
        self.currcon.pack_forget()
        for cti in self.coninfos:
            if (newbutton == cti.contype):
                self.currcon = cti;
                cti.pack()
                pass
            pass
        return

    def FillinConn(self):
        if (not self.enable):
            if (not self.enablevar.get()):
                return None
            pass
        return self.currcon.SetupArgs()

    def Cleanup(self):
        for cti in self.coninfos:
            cti.Cleanup()
            pass
        self.currcon = None
        return

    pass

class OpenDomainDialog(Tix.Toplevel):
    def __init__(self, mainhandler):
        Tix.Toplevel.__init__(self)
        self.title("Domain Creation Dialog")
        
        self.mainhandler = mainhandler

        self.name = Tix.LabelEntry(self, label="Domain name")
        self.name.pack(side=Tix.TOP, fill=Tix.X, expand=1)

        bbox = Tix.ButtonBox(self)
        bbox.add("cancel", text="Cancel", command=lambda w=self: w.cancel())
        bbox.add("ok", text="Ok", command=lambda w=self: w.ok())
        bbox.pack(side=Tix.BOTTOM, fill=Tix.X, expand=1)

        self.status = gui_errstr.ErrStr(self)
        self.status.pack(side=Tix.BOTTOM, fill=Tix.X, expand=1)

        self.conn = [ ConnInfo(self, mainhandler),
                      ConnInfo(self, mainhandler, False) ]
        self.conn[0].pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        self.conn[1].pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        self.name.entry.focus()

        self.bind("<Destroy>", self.OnDestroy)

        return

    def OnDestroy(self, event):
        # This doesn't get cleaned up properly by Python til after
        # exit, but we need to make sure the args get freed now, or
        # it won't be freed until after the OS handler is destroyed.
        for c in self.conn:
            c.Cleanup()
            pass
        return
    
    def cancel(self):
        self.destroy()
        return

    def ok(self):
        self.status.SetError("")
        name = str(self.name.entry.get())
        if (name == ""):
            self.status.SetError("No name specified")
            return
        try:
            args = [ self.conn[0].FillinConn() ]
            arg = self.conn[1].FillinConn()
            if (arg != None):
                args.append(arg);
                pass
            pass
        except Exception, e:
            self.status.SetError("Error handling connection arguments")
            import sys, traceback
            t, v, b = sys.exc_info()
            bl = traceback.format_tb(b)
            b = ""
            for x in bl:
                b += "\n" + x
            self.mainhandler.log("EINF", "Connection Argument Handling error: "
                                 + str(t) + ":" + str(v) + ":" + b)
            return
        domain_id = OpenIPMI.open_domain3(name, [], args, None, None)
        if (domain_id == None):
            self.status.SetError("Error opening domain")
            return

        self.destroy()
        return

    pass



# _sel.py
#
# openipmi GUI handling for SEL data
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import sys
import gui_list
import gui_popup
import _misc

class EventData:
    def __init__(self, slist, ev, has_second_data):
        self.slist = slist
        self.ev = ev
        self.second_key = None
        return

    def HandleMenu(self, event, idx, point):
        gui_popup.popup(self.slist, event,
                        [ ("Delete", self.delete, idx) ],
                        point)
        return

    def delete(self, idx):
        self.slist.DelItem(self.key)
        if (self.second_key != None):
            self.slist.DelItem(self.second_key)
            pass
        self.ev.delete()
        return
    
    pass

class EventInfo:
    def __init__(self):
        self.sensor = None
        self.val = None
        return

    def threshold_event_cb(self, sensor, event_spec, raw_set, raw,
                           value_set, value, event):
        self.sensor = sensor.get_name()
        self.val = event_spec;
        if (value_set):
            self.val += str(value)
            pass
        if (raw_set):
            self.val += '(' + str(raw) + ')'
            pass
        return

    def discrete_event_cb(self, sensor, event_spec, severity, old_severity,
                          event):
        self.sensor = sensor.get_name()
        self.val = (event_spec + ' ' + str(severity) +
                    '(' + str(old_severity) + ')')
        return

    pass

class SELDisplay(gui_list.List):
    def __init__(self, o, type):
        self.o = o
        self.type = type
        self.numevents = 0
        gui_list.List.__init__(self, "SEL for " + o.get_name(),
                               [ ("RecNum", 64), ("Type", 40),
                                 ("Time/Sensor", 300), ("Data", 400) ])

        self.events = [ ]
        
        ev = o.first_event()
        while (ev != None):
            self.AddEvent(ev)
            ev = o.next_event(ev)
            pass

        self.AfterDone()
        return

    def AddEvent(self, ev):
        evinfo = EventInfo()
        ev.call_handler(evinfo)

        data = EventData(self, ev, evinfo.sensor != None)
        self.events.append(data)
        data.key = self.Append(str(ev.get_record_id()),
                               [ str(ev.get_type()), str(ev.get_timestamp()),
                                 _misc.HexArrayToStr(ev.get_data()) ],
                               data)
        if (evinfo.sensor):
            # Can only delete the using the first item.
            data.second_key = self.Append("", [ "", evinfo.sensor, evinfo.val ],
                                          data)
            pass
        return
        
    def ok(self):
        self.Close()
        return

    def clear(self):
        for data in self.events:
            data.ev.delete()
            pass
        self.events = [ ]
        self.DeleteAllItems()
        return

    def do_on_close(self):
        self.events = None
        return

    pass

class DomainSELDisplay(SELDisplay):
    def __init__(self, domain_id):
        self.domain_id = domain_id
        self.init = True
        domain_id.to_domain(self)
        return

    def do_on_close(self):
        self.init = False
        self.domain_id.to_domain(self)
        SELDisplay.do_on_close(self)
        return

    def domain_cb(self, domain):
        if (self.init):
            SELDisplay.__init__(self, domain, "domain")
            domain.add_event_handler(self)
        else:
            domain.remove_event_handler(self)
            pass
        return

    def event_cb(self, domain, ev):
        self.AddEvent(ev)
        return

    pass

class MCSELDisplay(SELDisplay):
    def __init__(self, mc_id):
        self.init = True
        self.mc_id = mc_id
        mc_id.to_mc(self)
        return

    def do_on_close(self):
        self.init = False
        self.mc_id.to_mc(self)
        SELDisplay.do_on_close(self)
        return
    
    def mc_cb(self, mc):
        domain = mc.get_domain()
        if self.init:
            SELDisplay.__init__(self, mc, "MC")
            domain.add_event_handler(self)
        else:
            domain.remove_event_handler(self)
            pass
        return

    def event_cb(self, domain, ev):
        mc_id = ev.get_mc_id()
        if (mc_id.cmp(self.mc_id) == 0):
            self.AddEvent(ev)
            pass
        return

    pass



#!/usr/bin/python

# sample
#
# A sample file that uses most of the python/OpenIPMI interface
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2010 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import sys
import OpenIPMI

# Used to count things waiting for shutdown
stop_count = 1
main_handler = None

class MC_Nameget:
    def __init__(self):
        self.name = None
        pass

    def mc_cb(self, mc):
	self.name = mc.get_name()
        pass
    pass

threshold_list = ("ln", "lc", "lr", "un", "uc", "ur")
low_high = ("l", "h")
act_deact = ("a", "d")

class Handlers:
    def __init__(self, name):
        self.name = name
        return

    def event_cb(self, domain, event):
	mcid = event.get_mc_id()

	name = MC_Nameget()
	mcid.to_mc(name)
        name = name.name
	data = event.get_data()

	print ("Got event: " + name + " " + str(event.get_record_id()) +
               " " + str(event.get_type()) + " " + str(event.get_timestamp()))
        datastr = ""
        for b in data:
            datastr += " %2.2x" % b
            pass
	print "  Data:" + datastr
        return

    def entity_presence_cb(self, entity, present, event):
	print "Entity " + entity.get_name() + " presence is " + str(present)

        if event:
	    self.event_cb(entity.get_domain(), event)
            pass
        return

    def threshold_event_cb(self, sensor, event_spec, raw_set, raw,
                           value_set, value, event):
	print "Sensor " + sensor.get_name() + " got event " + event_spec
        if raw_set:
	    print "  raw value = " + str(raw)
            pass
        if value_set:
	    print "  value = " + str(value)
            pass
        if event:
            entity = sensor.get_entity()
	    self.event_cb(entity.get_domain(), event)
            pass
        return

    def discrete_event_cb(self, sensor, event_spec, severity, old_severity,
                          event):
	print "Sensor " + sensor.get_name() + " got event " + event_spec
	print "  severity = " + str(severity) + " was = " + str(old_severity)
        if event:
            entity = sensor.get_entity()
	    self.event_cb(entity.get_domain(), event)
            pass
        return

    def entity_sensor_update_cb(self, op, entity, sensor):
	print op + " sensor " + sensor.get_name()
        if ((op == "added") or (op == "changed")):
	    print "  lun = " + str(sensor.get_lun())
	    print "  num = " + str(sensor.get_num())
	    print "  sensor_type_string = " + sensor.get_sensor_type_string()
	    print "  sensor_type = " + str(sensor.get_sensor_type())
	    print("  event_reading_type_string = " +
                  sensor.get_event_reading_type_string())
	    print("  event_reading_type = "
                  + str(sensor.get_event_reading_type()))
	    print "  entity_id = " + str(sensor.get_entity_id())
	    print "  entity_instance = " + str(sensor.get_entity_instance())
	    print("  sensor_init_scanning = " +
                   str(sensor.get_sensor_init_scanning()))
	    print("  sensor_init_events = "
                  + str(sensor.get_sensor_init_events()))
	    print("  sensor_init_thresholds = " +
                  str(sensor.get_sensor_init_thresholds()))
	    print("  sensor_init_hysteresis = " +
                  str(sensor.get_sensor_init_hysteresis()))
	    print("  sensor_init_type = " +
                  str(sensor.get_sensor_init_type()))
	    print("  sensor_init_pu_events = " +
                  str(sensor.get_sensor_init_pu_events()))
	    print("  sensor_init_pu_scanning = " +
                  str(sensor.get_sensor_init_pu_scanning()))
	    print("  ignore_if_no_entity = " +
                  str(sensor.get_ignore_if_no_entity()))
	    print("  supports_auto_rearm = " +
                  str(sensor.get_supports_auto_rearm()))
	    print "  event_support = " + str(sensor.get_event_support())
	    print "  sensor_direction = " + str(sensor.get_sensor_direction())
	    print "  oem1 = " + str(sensor.get_oem1())
	    print "  sensor_id = " + sensor.get_sensor_id()

	    if (sensor.get_event_reading_type()
                == OpenIPMI.EVENT_READING_TYPE_THRESHOLD):
		print "  rate_unit_string = " + sensor.get_rate_unit_string()
		print "  rate_unit = " + str(sensor.get_rate_unit())
		print("  base_unit_string = " +
                      sensor.get_base_unit_string())
		print "  base_unit = " + str(sensor.get_base_unit())
		print("  modifier_unit_string = " +
		       sensor.get_modifier_unit_string())
		print "  modifier_unit = " + str(sensor.get_modifier_unit())
		print("  modifier_unit_use = " +
                      str(sensor.get_modifier_unit_use()))
		print "  percentage = " + str(sensor.get_percentage())
		print("  threshold_access = " +
		      str(sensor.get_threshold_access()))
		print("  hysteresis_support = " +
		      str(sensor.get_hysteresis_support()))

                val = [ 0.0 ]
                if sensor.get_normal_min_specified():
		    rv = sensor.get_normal_min(val)
                    if rv:
                        print "***Error getting normal min: " + rv
		    else:
			print "  normal_min = " + str(val[0])
                        pass
                    pass

		if sensor.get_normal_max_specified():
		    rv = sensor.get_normal_max(val)
                    if rv:
                        print "***Error getting normal max: " + rv
		    else:
			print "  normal_max = " + str(val[0])
                        pass
                    pass

                if sensor.get_nominal_reading_specified():
		    rv = sensor.get_nominal_reading(val)
                    if rv:
			print "***Error getting nominal reading: " + str(rv)
		    else:
			print "  nominal_reading = " + str(val[0])
                        pass
                    pass
                
		rv = sensor.get_sensor_max(val)
                if rv:
		    print "***Error getting sensor max: " + str(rv)
		else:
		    print "  sensor_max = " + str(val[0])
                    pass
                pass

		rv = sensor.get_sensor_min(val)
                if rv:
		    print "***Error getting sensor min: " + str(rv)
		else:
		    print "  sensor_min = " + str(val[0])
                    pass
                pass

		rv = sensor.get_tolerance(128, val)
                if rv == 0:
		    print "  tolerance at 128 is " + str(val[0])
                    pass

		rv = sensor.get_accuracy(128, val)
                if rv == 0:
		    print "  accuracy at 128 is " + str(val[0])
                    pass

                for i in threshold_list:
		    supported = [ 0 ]
		    settable = [ 0 ]
		    readable = [ 0 ]

		    rv = sensor.threshold_reading_supported(i, supported)
                    if rv:
			print("***Error getting supported for threshold %s: %s"
                              % (i, str(rv)))
                        pass
                    supported = supported[0]

                    if not supported:
			continue

		    tstr = "  Threshold '" + i + "' supported:"

		    rv = sensor.threshold_settable(i, settable)
                    if rv:
			print("***Error getting settable for threshold %s: %s"
                              % (i, str(rv)))
                        pass
                    settable = settable[0]
                    if settable:
			tstr += " settable"
                        pass

		    rv = sensor.threshold_readable(i, readable)
                    if rv:
			print("***Error getting readable for threshold %s: %s"
                              % (i, str(rv)))
                        pass
                    readable = readable[0]
                    if readable:
			tstr += " readable"
                        pass
		    print tstr

		    tstr = "    Supports events:"
                    for j in low_high:
                        for k in act_deact:
			    e = i + j + k
			    s = [ 0 ]
			    rv = sensor.threshold_event_supported(e, s)
                            if rv:
				print ("***Error getting ev support for event "
                                       + e + ": " + str(rv))
			    elif s[0]:
				tstr += " " + e
                                pass
                            pass
                        pass
		    print tstr
                    pass
                pass
            else:
                for i in range(0, 14):
		    a_supported = [ 0 ]
		    d_supported = [ 0 ]
		    readable = [ 0 ]

		    rv = sensor.discrete_event_readable(i, readable)
                    if rv:
			print ("***Error getting readable for offset %d: %s"
                               % (i, str(rv)))
                        pass
                    readable = readable[0]
                    
		    rv = sensor.discrete_event_supported(str(i) + "a",
                                                         a_supported)
                    if rv:
			print ("***Error getting a supported for offset %d: %s"
                               % (i, str(rv)))
                        pass
                    a_supported = a_supported[0]

		    rv = sensor.discrete_event_supported(str(i) + "d",
                                                         d_supported)
                    if rv:
			print ("***Error getting d supported for offset %s: %s"
                               % (i, str(rv)))
                        pass
                    d_supported = d_supported[0]

                    if readable or a_supported or d_supported:
			s = sensor.reading_name_string(i)
			tstr = "  Offset %d (%s) supported:" % (i, s)

                        if readable:
			    tstr += " readable"
                            pass
                        if a_supported:
			    tstr += " assert"
                            pass
                        if d_supported:
			    tstr += " deassert"
                            pass
                        print tstr
                        pass
                    pass
                pass
	    
            if op == "added":
		rv = sensor.add_event_handler(self)
                if rv:
		    print "***Unable to add event handler: " + str(rv)
                    pass
                pass
            pass
        elif op == "deleted":
            rv = sensor.remove_event_handler(self)
            if rv:
		print "***Unable to remove event handler: " + str(rv)
                pass
            pass
        return

    def control_get_val_cb(self, control, err, vals):
	tstr = "Control %s err = %s, values = " % (control.get_name(),str(err))
        for v in vals:
            tstr += " %d" % v
            pass
	print tstr
        return

    def control_get_light_cb(self, control, err, vals):
	print("Control %s err = %s, light value = %s"
              % (control.get_name(), str(err), vals))
        return

    def control_get_id_cb(self, control, err, val):
	tstr = ("Control %s err = %s, id value = "
                % (control.get_name(), str(err)))
        for v in vals:
            tstr += " %2.2x" % v
            pass
        print tstr
        return

    def control_event_val_cb(self, control, event, num, valids, vals):
	tstr = "Control " + control.get_name() + " event, values ="
        for i in range(0, len(valids)):
            if valids[i]:
                v = "valid"
            else:
                v = "invalid"
                pass
            tstr += " (%s, %d)" % (v, vals[i])
            pass
        print tstr
        return

    def entity_control_update_cb(self, op, entity, control):
	use_setting = 0

	print op + " control " + control.get_name()
        if ((op == "added") or (op == "changed")):
	    t = control.get_type()
	    print "  type = " + str(t)
	    print "  entity_id = " + str(control.get_entity_id())
	    print "  entity_instance = " + str(control.get_entity_instance())
	    print "  settable = " + str(control.is_settable())
	    print "  readable = " + str(control.is_readable())
	    print("  ignore_if_no_entity = " +
		  str(control.get_ignore_if_no_entity()))
	    print "  control_id = " + str(control.get_control_id())
	    print "  has_events = " + str(control.has_events())
	    num_vals = control.get_num_vals()
	    print "  num_vals = " + str(num_vals)

            if t == OpenIPMI.CONTROL_LIGHT:
		use_setting = control.light_set_with_setting()
                pass

	    if use_setting:
		print "  light controlled with setting"
		tstr = "  Colors: "
                for i in range(0, OpenIPMI.CONTROL_NUM_COLORS):
                    if (control.light_is_color_supported(i)):
			tstr += " " + OpenIPMI.color_string(i) + ("(%d)" % i)
                        pass
                    pass
		print tstr
		print("  light_has_local_control = " +
		      str(control.light_has_local_control()))
		rv = control.ipmi_control_get_light(self)
                if rv:
		    print "***Error getting light: " + str(rv)
                    pass
                pass
	    elif t == OpenIPMI.CONTROL_IDENTIFIER:
		print("  identifier_get_max_length = " +
		      str(control.identifier_get_max_length()))
		rv = control.identifier_get_val(self)
                if rv:
		    print "***Error getting control id: " + str(rv)
                    pass
                pas
            elif t == OpenIPMI.CONTROL_LIGHT:
                for i in range(0, num_vals) :
                    n = control.get_num_light_values(i)
                    print "  Light %d" % i
                    for j in range (0, n):
                        print "    Value %d" % j
                        o = control.get_num_light_transitions(i, j)
                        for k in range (0, o):
                            tstr = "    Transition %d:" & k
                            v = control.get_light_color(i, j, k)
                            tstr += " " + OpenIPMI.color_string(v)
                            v = control.get_light_color_time(i, j, k)
                            tstr += " time(" + str(v) + ")"
                            pass
                        pass
                    pass
                pass

            if op == "added":
		rv = control.add_event_handler(self)
                if rv:
		    print "***Error adding control event handler: " + str(rv)
                    pass
                pass
            pass
        elif op == "deleted":
	    rv = control.remove_event_handler(self)
            if rv:
		print "***Error removing control event handler: " + str(rv)
                pass
            pass
        return
    
    def print_multirecord(self, node, indent):
	name = [ "" ]
	t = [ "" ]
	value = [ "" ]
	sub_node = [ None ]
    
	i = 0
        while True:
	    rv = node.get_field(i, name, t, value, sub_node)
            if rv == OpenIPMI.einval:
		return
	    i += 1
            if rv:
		continue

	    print "%s%s, %s, %s" % (indent, name[0], t[0], value[0])
            if t == "subnode":
		self.print_multirecord(subnode[0], indent + "  ")
                pass
            pass
        return

    def entity_fru_update_cb(self, op, entity, fru):
	print op + " fru for " + entity.get_name()
        if ((op == "added") or (op == "changed")):
	    i = fru.str_to_index("internal_use_version")
            if i == -1:
		print "*** FRU string 'internal_use_version' was invalid"
	    else:
		print "  internal_use_version index is %d" % i
                pass
	    i = fru.str_to_index("blah blah")
            if i != -1:
		print "*** FRU string 'blah blah' was valid"
                pass

	    i = 0
	    j = [ 0 ]
	    k = j[0]
	    rv = fru.get(i, j)
            while rv:
                v = rv.split(" ", 2)
                if len(v) > 2:
                    name = v[0]
                    t = v[1]
		    print "  %s, %s, %s" % (name, t, v[2])
                    pass
                if j[0] != k:
                    if j[0] == -1:
			i += 1
			j = [ 0 ]
                        pass
                    pass
                else:
		    i += 1
		    j = [ 0 ]
                    pass

		k = j[0]
		rv = fru.get(i, j)
                pass
	    j = fru.get_num_multi_records()
	    print "%d multirecords" % j
            for i in range(0, j):
		name = [ "" ]
		sub_node = [ None ]
		rv = fru.multi_record_get_root_node(i, name, sub_node)
                if rv:
		    print "Multirecord %d has no decoder" % i
		    continue
		print "Multirecord %d (%s)" % (i, name[0])
		self.print_multirecord(sub_node[0], "  ")
                pass
            pass
        return

    def entity_hot_swap_update_cb(self, entity, old_state, new_state, event):
	print("Hot swap change for %s was %s now %s"
              % (entity.get_name(), old_state, new_state))
        if event:
	    self.event_cb(entity.get_domain(), event)
            pass
        return

    def entity_update_cb(self, op, domain, entity):
	print op + " entity " + entity.get_name()
        if ((op == "added") or (op == "changed")):
	    print "  type = " + entity.get_type()
	    print "  is_fru = " + str(entity.is_fru())
	    print "  entity_id = " + str(entity.get_entity_id())
	    print "  entity_instance = " + str(entity.get_entity_instance())
	    print("  device_channel = " +
		  str(entity.get_entity_device_channel()))
	    print("  device_address = " +
		  str(entity.get_entity_device_address()))
	    print("  presence_sensor_always_there = " +
		   str(entity.get_presence_sensor_always_there()))
	    print "  channel = " + str(entity.get_channel())
	    print "  lun = " + str(entity.get_lun())
	    print "  oem = " + str(entity.get_oem())
	    print "  access_address = " + str(entity.get_access_address())
	    print "  private_bus_id = " + str(entity.get_private_bus_id())
	    print "  device_type = " + str(entity.get_device_type())
	    print "  device_modifier = " + str(entity.get_device_modifier())
	    print "  slave_address = " + str(entity.get_slave_address())
	    print "  is_logical_fru = " + str(entity.get_is_logical_fru())
	    print "  fru_device_id = " + str(entity.get_fru_device_id())
	    print("  ACPI_system_power_notify_required = " +
		  str(entity.get_ACPI_system_power_notify_required()))
	    print("  ACPI_device_power_notify_required = " +
		  str(entity.get_ACPI_device_power_notify_required()))
	    print("  controller_logs_init_agent_errors = " +
		  str(entity.get_controller_logs_init_agent_errors()))
	    print("  log_init_agent_errors_accessing = " +
		  str(entity.get_log_init_agent_errors_accessing()))
	    print "  global_init = " + str(entity.get_global_init())
	    print "  chassis_device = " + str(entity.get_chassis_device())
	    print "  bridge = " + str(entity.get_bridge())
	    print("  IPMB_event_generator = " +
		  str(entity.get_IPMB_event_generator()))
	    print("  IPMB_event_receiver = " +
		  str(entity.get_IPMB_event_receiver()))
	    print("  FRU_inventory_device = " +
		  str(entity.get_FRU_inventory_device()))
	    print "  SEL_device = " + str(entity.get_SEL_device())
	    print("  SDR_repository_device = " +
		  str(entity.get_SDR_repository_device()))
	    print "  sensor_device = " + str(entity.get_sensor_device())
	    print "  address_span = " + str(entity.get_address_span())
	    print "  dlr_id = " + str(entity.get_dlr_id())
	    print "  present = " + str(entity.is_present())
	    print "  hot_swappable = " + str(entity.is_hot_swappable())
            if op == "added":
		rv = entity.add_presence_handler(self)
                if rv:
		    print "***Unable to add presence handler: " + str(rv)
                    pass
		rv = entity.add_sensor_update_handler(self)
                if rv:
		    print "***Unable to add sensor update handler: " + str(rv)
                    pass
		rv = entity.add_control_update_handler(self)
                if rv:
		    print "***Unable to add control update handler: " + str(rv)
                    pass
		rv = entity.add_hot_swap_handler(self)
                if rv:
		    print "***Unable to add hot-swap handler: " + str(rv)
                    pass
		rv = entity.add_fru_update_handler(self)
                if rv:
		    print "***Unable to add fru handler: " + str(rv)
                    pass
                pass
            pass
        elif op == "deleted":
	    rv = entity.remove_presence_handler(self)
	    if rv:
                print "***Unable to remove presence handler: " + str(rv)
                pass
	    rv = entity.remove_sensor_update_handler(self)
	    if rv:
                print "***Unable to remove sensor update handler: " + str(rv)
                pass
	    rv = entity.remove_control_update_handler(self)
	    if rv:
                print "***Unable to remove control update handler: " + str(rv)
                pass
	    rv = entity.remove_hot_swap_handler(self)
            if rv:
		print "***Unable to remove hot-swap handler: " + str(rv)
                pass
	    rv = entity.remove_fru_update_handler(self)
            if rv:
		print "***Unable to add fru handler: " + str(rv)
                pass
            pass
        return

    def mc_active_cb(self, mc, active):
	print "MC " + mc.get_name() + " active set to " + str(active)
        return

    def mc_events_enable_cb(self, mc, err, domain):
        global stop_count
	print "Events enabled for " + mc.get_name() + " err = " + str(err)
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def mc_reread_sel_cb(self, mc, err):
        global stop_count
	domain = mc.get_domain()

	print "SEL reread for " + mc.get_name() + " err = " + str(err)
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def mc_reread_sensors_cb(self, mc, err):
        global stop_count
	domain = mc.get_domain()

	print "Sensors reread for " + mc.get_name() + " err = " + str(err)
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def mc_get_sel_time_cb(self, mc, err, time):
        global stop_count
	domain = mc.get_domain()

	print("SEL time for " + mc.get_name() + " is " + str(time) + ", err = "
              + str(err))
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def mc_update_cb(self, op, domain, mc):
        global stop_count
	print op + " MC " + mc.get_name()
        if ((op == "added") or (op == "changed")):
	    print("  provides_device_sdrs = " +
		  str(mc.provides_device_sdrs()))
	    print "  device_available = " + str(mc.device_available())
	    print "  chassis_support = " + str(mc.chassis_support())
	    print "  bridge_support = " + str(mc.bridge_support())
	    print("  ipmb_event_generator_support = " +
		  str(mc.ipmb_event_generator_support()))
	    print("  ipmb_event_receiver_support = " +
                  str(mc.ipmb_event_receiver_support()))
	    print("  fru_inventory_support = " +
		  str(mc.fru_inventory_support()))
	    print("  sel_device_support = " +
		  str(mc.sel_device_support()))
	    print("  sdr_repository_support = " +
		  str(mc.sdr_repository_support()))
	    print("  sensor_device_support = " +
		  str(mc.sensor_device_support()))
	    print "  device_id = " + str(mc.device_id())
	    print "  device_revision = " + str(mc.device_revision())
	    print "  major_fw_revision = " + str(mc.major_fw_revision())
	    print "  minor_fw_revision = " + str(mc.minor_fw_revision())
	    print "  major_version = " + str(mc.major_version())
	    print "  minor_version = " + str(mc.minor_version())
	    print "  manufacturer_id = " + str(mc.manufacturer_id())
	    print "  product_id = " + str(mc.product_id())
	    print "  aux_fw_revision = " + str(mc.aux_fw_revision())
	    print "  is_active = " + str(mc.is_active())
	    print "  get_events_enable = " + str(mc.get_events_enable())
	    print "  sel_count = " + str(mc.sel_count())
	    print "  sel_entries_used = " + str(mc.sel_entries_used())
	    print("  sel_get_major_version = " +
		  str(mc.sel_get_major_version()))
	    print("  sel_get_minor_version = " +
		  str(mc.sel_get_minor_version()))
	    print("  sel_get_num_entries = " +
		  str(mc.sel_get_num_entries()))
	    print "  sel_get_free_bytes = " + str(mc.sel_get_free_bytes())
	    print "  sel_get_overflow = " + str(mc.sel_get_overflow())
	    print("  sel_get_supports_delete_sel = " +
		  str(mc.sel_get_supports_delete_sel()))
	    print("  sel_get_supports_partial_add_sel = " +
		  str(mc.sel_get_supports_partial_add_sel()))
	    print("  sel_get_supports_reserve_sel = " +
		  str(mc.sel_get_supports_reserve_sel()))
	    print("  sel_get_supports_get_sel_allocation = " +
		  str(mc.sel_get_supports_get_sel_allocation()))
	    print("  sel_get_last_addition_timestamp = " +
		  str(mc.sel_get_last_addition_timestamp()))
	    print("  get_sel_rescan_time = " +
		  str(mc.get_sel_rescan_time()))

	    stop_count += 2
	    rv = mc.set_events_enable(1, self)
	    if rv:
		print "***Error enabling MC events: " + str(rv)
                stop_count -= 1
                pass

	    stop_count += 1
	    rv = mc.reread_sensors(self)
            if rv:
		print "***Error rereading MC sensors: " + str(rv)
		stop_count -= 1
                pass

	    mc.set_sel_rescan_time(5)

	    stop_count += 1
	    rv = mc.reread_sel(self)
            if rv:
		print "***Error rereading MC SEL: " + str(rv)
		stop_count -= 1
                pass

	    stop_count += 1
	    rv = mc.get_current_sel_time(self)
            if rv:
		print "***Error getting current MC SEL time: " + str(rv)
		stop_count -= 1
                pass

	    # Stop count for this incremented at first.
	    rv = mc.send_command(0, 10, 0x43,
                                 [ 0, 0, 0, 0, 0, 0xff ], self)
            if rv:
		print "***Unable to send mc command: " + str(rv)
		stop_count -= 1
                pass

            if op == "added":
		rv = mc.add_active_handler(self)
                if rv:
		    print "***Unable to add active handler: " + str(rv)
                    pass
                pass
            pass
        elif op == "deleted":
            rv = mc.remove_active_handler(self)
            if rv:
                print "***Unable to remove active handler: " + str(rv)
                pass
            pass
        return

    def domain_addr_cmd_cb(self, domain, addr, lun, netfn, cmd, data):
	print "Got message from " + domain.get_name()
	print " addr = " + addr
	print " lun=%d, netfn=%d, cmd=%d" % (lun, netfn, cmd)
	tstr = " data:"
        for b in data:
	    tstr += " %2.2x" % b
            pass
	print tstr
        return

    def mc_cmd_cb(self, mc, netfn, cmd, data):
        global stop_count
	print "Got message from " + mc.get_name()
	print " netfn=%d, cmd=%d" % (netfn, cmd)
	tstr = " data:"
        for b in data:
	    tstr += " %2.2x" % b
            pass
	print tstr
	stop_count -= 1
        if stop_count == 0:
	    mc.get_domain().close(main_handler)
            pass
        return

    def conn_change_cb(self, domain, err, conn_num, port_num, still_connected):
	print "Open done (%s): %s" % (self.name, domain.get_name())
        if self.name == "hello":
	    i = Handlers("goodbye")
	    rv = domain.add_connect_change_handler(i)
            if rv:
		print "Unable to add connect change handler: " + str(rv)
                pass
	    rv = domain.remove_connect_change_handler(self)
            if rv:
		print "Unable to remove connect change handler: " + str(rv)
                pass

	    rv = domain.add_entity_update_handler(i)
            if rv:
		print "Unable to add entity updated handler: " + str(rv)
                pass
	    rv = domain.add_mc_update_handler(i)
            if rv:
		print "Unable to add mc updated handler: " + str(rv)
                pass

	    rv = domain.send_command_addr("smi 15 ", 0, 6, 1, [], self)
            if rv:
		print "Unable to send domain command (1): " + str(rv)
                pass

	    rv = domain.add_event_handler(self)
            if rv:
		print "Unable to add event handler: " + str(rv)
                pass
	else:
	    rv = domain.send_command_addr("ipmb 0 32", 0, 10, 0x43,
                                          [ 0, 0, 0, 0, 0, 0xff ], self)
            if rv:
		print "Unable to send domain command (2): " + str(rv)
                pass
            pass
        return

    def domain_up_cb(self, domain):
        global stop_count
	print "Domain up: " + domain.get_name()
	print "  type = " + domain.get_type()
	print "  sel_rescan_type = " + str(domain.get_sel_rescan_time())
	print "  ipmb_rescan_type = " + str(domain.get_ipmb_rescan_time())
	domain.set_sel_rescan_time(5)
	domain.set_ipmb_rescan_time(20)
	domain.iterate_entities(self)
	domain.iterate_mcs(self)

	event = domain.first_event()
        while event:
	    self.event_cb(domain, event)
	    event = domain.next_event(event)
            pass

	# stop count is incremented at domain startup, don't need another.
	stop_count += 1
	rv = domain.start_ipmb_mc_scan(0, 0x20, 0x20, self)
        if rv:
	    print "Error starting IPMB scan: " + str(rv)
	    stop_count -= 1
            pass

	rv = domain.reread_sels(self)
        if rv:
	    print "Error starting IPMB scan: " + str(rv)
	    stop_count -= 1
            pass
        return

    def domain_reread_sels_cb(self, domain, err):
        global stop_count
	print "SEL rescan done for " + domain.get_name() + " err=" + str(err)
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def domain_close_done_cb(self):
	self.name = "done"
        return

    def domain_ipmb_mc_scan_cb(self, domain, err):
        global stop_count
	print "IPMB scan done for " + domain.get_name() + " err=" + str(err)
	stop_count -= 1
        if stop_count == 0:
	    domain.close(main_handler)
            pass
        return

    def entity_iter_entities_cb(self, relative, entity):
	print "    " + entity.get_name()
        return

    def entity_iter_sensors_cb(self, entity, sensor):
	print "Sensor: " + sensor.get_name()

    def entity_iter_controls_cb(self, entity, control):
	print "Control: " + control.get_name()

    def domain_iter_entity_cb(self, domain, entity):
	print "Entity: " + entity.get_name()
        if entity.is_child():
	    print "  Parents:"
	    entity.iterate_parents(self)
            pass
        if entity.is_parent():
	    print "  Children:"
	    entity.iterate_children(self)
            pass

	entity.iterate_sensors(self)
	entity.iterate_controls(self)
        return

    def domain_iter_mc_cb(self, domain, mc):
	print "MC: " + mc.get_name()

	event = mc.first_event()
        while event:
	    self.event_cb(mc.get_domain(), event)
	    event = mc.next_event(event)
            pass
        return

    def log(self, level, log):
	print level + ": " + log
        return

    pass

OpenIPMI.enable_debug_malloc()
OpenIPMI.init_posix()

main_handler = Handlers("hello")

OpenIPMI.set_log_handler(main_handler)

a = OpenIPMI.open_domain2("test", sys.argv[1:], main_handler, main_handler)
if not a:
    print "open failed"
    sys.exit(1)
    pass

while main_handler.name != "done":
    OpenIPMI.wait_io(1000)
    pass

OpenIPMI.shutdown_everything()
print "done"
sys.exit(0)



# gui_treelist.py
#
# A tree/list widget
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix
import gui_errstr
import gui

class TreeList(Tix.Toplevel):
    def __init__(self, name, root, columns):
        Tix.Toplevel.__init__(self)
        self.title(name)

        self.numcolumns = len(columns)
        
        stree = Tix.Tree(self,
                         options=("hlist.columns " + str(self.numcolumns)
                                  + " hlist.itemtype text"
                                  + " hlist.header 1"
                                  + " hlist.selectForeground black"
                                  + " hlist.selectBackground beige"),
                                  width=500, height=500)
        self.stree = stree
        tree = stree.hlist
        i = 0
        for c in columns:
            tree.header_create(i, text=c[0])
            tree.column_width(i, c[1])
            i += 1
            pass
        stree.pack(side=Tix.TOP, fill=Tix.BOTH, expand=1)
        
        self.errstr = gui_errstr.ErrStr(self)
        self.errstr.pack(side=Tix.TOP, fill=Tix.X, expand=1)
        
        bbox = Tix.ButtonBox(self)
        if hasattr(self, "ok"):
            bbox.add("ok", text="Ok", command=self.ok)
            pass
        if hasattr(self, "save"):
            bbox.add("save", text="Save", command=self.save)
            pass
        if hasattr(self, "cancel"):
            bbox.add("cancel", text="Cancel", command=self.cancel)
            pass
        if hasattr(self, "refresh"):
            bbox.add("refresh", text="Refresh", command=self.refresh)
            pass
        if hasattr(self, "clear"):
            bbox.add("clear", text="Clear", command=self.clear)
            pass
        bbox.pack(side=Tix.TOP, fill=Tix.X, expand=1)

        tree.bind("<Button-3>", self.TreeMenu)

        self.bind("<Destroy>", self.OnDestroy)
        
        self.bind("<MouseWheel>", self.Wheel)
        if (gui.winsys == "x11"):
            self.bind("<Button-4>", self.ButtonUp)
            self.bind("<Button-5>", self.ButtonDown)
            pass

        self.treeroot = ""
        self.tree = tree
        self.treehash = { }
        self.currkey = 0
        return

    def Wheel(self, event):
        self.tree.yview("scroll", -(event.delta / 20), "units")
        return
    
    def ButtonUp(self, event):
        event.delta = 120
        self.Wheel(event);
        return
    
    def ButtonDown(self, event):
        event.delta = -120
        self.Wheel(event);
        return
    
    def ExpandItem(self, item):
        children = self.stree.hlist.info_children(item)
        for child in children:
            self.stree.open(child)
            self.ExpandItem(child)
            pass
        return
        
    def Close(self):
        self.destroy()
        return

    def OnDestroy(self, event):
        if (hasattr(self, "do_on_close")):
            self.do_on_close()
            pass
        for i in self.treehash.values():
            if (hasattr(i, "do_on_close")):
                i.do_on_close()
                pass
            pass
        self.treehash = { }
        return

    def TreeMenu(self, event):
        w = event.widget
        key = w.nearest(event.y)
        data = self.treehash[key]
        if (data and hasattr(data, "HandleMenu")):
            data.HandleMenu(event, key, event)
            pass
        return

    def AfterDone(self):
        return

    def Append(self, node, name, values, data=None, before=None):
        hide = False
        if (node == ""):
            key = str(self.currkey)
        else:
            key = node + "." + str(self.currkey)
            mode = self.stree.getmode(node)
            if (mode == "none"):
                self.stree.setmode(node, "open")
                hide = True
                pass
            elif (mode == "open"):
                hide = True
                pass
            pass
        self.currkey += 1
        if (before != None):
            self.tree.add(key, text=name, before=before)
            pass
        else:
            self.tree.add(key, text=name)
            pass
        if (hide):
            self.tree.hide_entry(key)
            pass
        i = 1
        for v in values:
            if (v != None):
                self.tree.item_create(key, i, text=str(v))
            else:
                self.tree.item_create(key, i)
                pass
            i += 1
            pass
        for j in range(i, self.numcolumns):
            self.tree.item_create(key, j)
            pass
        self.treehash[key] = data
        return key

    def Remove(self, key):
        self.stree.hlist.delete_entry(key)
        del self.treehash[key]
        pass

    def RemoveAll(self):
        self.stree.hlist.delete_all()
        for i in self.treehash.values():
            if (hasattr(i, "do_on_close")):
                i.do_on_close()
                pass
            pass
        self.treehash = { }
        pass

    def SetColumn(self, node, value, colnum):
        self.tree.item_configure(node, colnum, text=value)
        return

    def GetColumn(self, node, colnum):
        return self.tree.item_cget(node, colnum, "-text")

    def SetError(self, str):
        self.errstr.SetError(str)
        return

    def add_data(self, parent, name, value, data=None, before=None):
        item = self.Append(parent, name, value, data, before);
        if (data != None):
            data.SetItem(item)
            pass
        return item

    pass



# _saveprefs.py
#
# Code to save/restore openipmi GUI preferences
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import xml.dom
import xml.dom.minidom
import _oi_logging
import os
import stat
import sys

taghash = { }

class RestoreHandler:
    def __init__(self, tag):
        taghash[tag] = self
        return

    def restore(self, attrlist):
        return

    pass


def save(objlist, file):
    domimpl = xml.dom.getDOMImplementation()
    doc = domimpl.createDocument(None, "IPMIPrefs", None)
    main = doc.documentElement
    for obj in objlist:
        elem = doc.createElement(obj.getTag())
        obj.SaveInfo(doc, elem)
        main.appendChild(elem)
        pass
    try:
        info = os.stat(file)
        pass
    except:
        # File doesn't exist, create it.
        try:
            fd = os.open(file, os.O_WRONLY | os.O_CREAT,
                         stat.S_IRUSR | stat.S_IWUSR)
            os.close(fd)
            pass
        except:
            _oi_logging.error("Unable to create startup file " + file)
            return
        pass
    try:
        f = open(file, 'w')
        doc.writexml(f, indent='', addindent='\t', newl='\n')
        pass
    except:
        _oi_logging.error("Unable to save startup file " + file)
        pass
    return

def restore(file):
    info = None
    try:
        info = os.stat(file)
    except:
        pass
    if (info):
        if ((info.st_mode & (stat.S_IRWXG | stat.S_IRWXO)) != 0):
            sys.exit("The file '" + file + "' is group or world"
                     + " accessible.  It contains passwords, and"
                     + " should be secure.  Not starting the GUI,"
                     + " please fix the problem first.")
            return
        pass
    try:
        doc = xml.dom.minidom.parse(file).documentElement
    except:
        pass
    else:
        for child in doc.childNodes:
            if (child.nodeType == child.ELEMENT_NODE):
                tag = child.nodeName
                if (tag in taghash):
                    taghash[tag].restore(child)
                    child = child.nextSibling
                    pass
                pass
            pass
        pass
    return



# gui_cmdwin.py
#
# openipmi GUI command window handling
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix
import xml.dom
import xml.dom.minidom
import OpenIPMI
import _saveprefs
import _misc
import os
import stat
import sys

init_history = [ ]

class CommandWindow(Tix.ScrolledText):
    def __init__(self, parent, ui):
        global init_history
        Tix.ScrolledText.__init__(self, parent)
        self.ui = ui
        self.currow = 0
        self.max_lines = 1000
        self.max_history = 100
        self.text.bind("<Key>", self.HandleChar)
        self.text.bind("<Control-Key>", self.HandleCtrlChar)
        self.text.insert("end", "> ")
        self.history = [ ]
        self.lasthist = 0
        for cmd in init_history:
            self.history.append(cmd[1])
            self.lasthist += 1
            pass
        self.history.append("")
        init_history = None
        self.currhist = self.lasthist

        self.cmdlang = OpenIPMI.alloc_cmdlang(self)
        self.indent = 0;
        self.cmd_in_progress = False

        self.bind("<Destroy>", self.OnDestroy)

        OpenIPMI.set_cmdlang_global_err_handler(self)
        return

    def global_cmdlang_err(self, objstr, location, errstr, errval):
        log = "Global cmdlang err: " + errstr;
        if (len(location) > 0) or (len(objstr) > 0):
            log += " at " + objstr + "(" + location + ")"
            pass
        log += ": " + errstr + " (" + str(errval) + ")"
        self.ui.new_log(log)
        return
    
    def OnDestroy(self, event):
        self.cmdlang = None
        return

    def cmdlang_down(self, cmdlang):
        self.indent += 2
        return
    
    def cmdlang_up(self, cmdlang):
        if (self.indent >= 2):
            self.indent -= 2
            pass
        return
    
    def HandleNewLines(self):
        lastline = int(self.text.index("end").split(".")[0]) - 1
        while (lastline > self.max_lines):
            self.text.delete("1.0", "2.0")
            lastline = int(self.text.index("end").split(".")[0]) - 1
            pass
        return

    def InsertString(self, string):
        (lastrow, lastcol) = self.text.index("end").split(".")
        lastrow = str(int(lastrow)-1)
        self.text.insert(lastrow + ".0", string)
        self.HandleNewLines()
        self.text.see("insert")
        return
    
    def cmdlang_done(self, cmdlang):
        err = cmdlang.get_err()
        if (err != 0):
            errtext = cmdlang.get_errstr()
            objstr = cmdlang.get_objstr()
            location = cmdlang.get_location()
            if (location == None):
                location = ""
                pass
            if (objstr == ""):
                str = ("error: %s: %s (0x%x, %s)\n"
                       % (location, errtext, err,
                          OpenIPMI.get_error_string(err)))
                pass
            else:
                str = ("error: %s %s: %s (0x%x, %s)\n"
                       % (location, objstr, errtext, err,
                          OpenIPMI.get_error_string(err)))
                pass
            self.InsertString(str)
            pass
        self.cmd_in_progress = False
        self.text.insert("end", "> ")
        return

    def cmdlang_out(self, cmdlang, name, value):
        if (cmdlang.is_help()):
            self.InsertString("%*s%s %s\n" % (self.indent, "", name, value))
            pass
        else:
            self.InsertString("%*s%s: %s\n" % (self.indent, "", name, value))
            pass
        return
    
    def cmdlang_out_binary(self, cmdlang, name, value):
        self.InsertString("%*s%s: %s\n" % (self.indent, "", name,
                                           _misc.HexArrayToStr(value)))
        return
    
    def cmdlang_out_unicode(self, cmdlang, name, value):
        self.InsertString("%*s%s:U: %s\n" % (self.indent, "", name,
                                             _misc.HexArrayToStr(value)))
        return
    
    def HandleNewHistory(self):
        self.history.append("")
        if (self.lasthist >= self.max_history):
            del self.history[0]
            pass
        else:
            self.lasthist += 1
            pass
        return
    
    def HandleCtrlChar(self, event):
        # This is here to catch the control characters and pass them
        # on so HandleChar() doesn't trap and throw them away.
        return
    
    def HandleChar(self, event):
        key = event.keysym
        if (key == "BackSpace"):
            # A key that will result in a backspace.  Make sure it
            # only occurs on the last line and not in the prompt area.
            if (self.cmd_in_progress):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            (currrow, currcol) = self.text.index("insert").split(".")
            if ((lastrow != currrow) or (int(currcol) <= 2)):
                # Ignore the keypress
                return "break"
            pass
        elif (key == "Delete"):
            # A key that will result in a deletion.  Make sure it
            # only occurs on the last line and not in the prompt area.
            if (self.cmd_in_progress):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            (currrow, currcol) = self.text.index("insert").split(".")
            if ((lastrow != currrow) or (int(currcol) <= 1)):
                # Ignore the keypress
                return "break"
            pass
        elif (key == "Return"):
            # Enter the command...
            if (self.cmd_in_progress):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            (currrow, currcol) = self.text.index("insert").split(".")
            if ((lastrow != currrow) or (int(currcol) <= 2)):
                # Ignore the keypress
                return "break"

            command = self.text.get(lastrow + ".2", lastrow + ".end")
            self.HandleNewLines();
            if (command != ""):
                self.text.insert("end", "\n")
                self.history[self.lasthist] = command
                self.HandleNewHistory()
                self.cmdlang.handle(str(command))
                pass
            else:
                self.text.insert("end", "\n> ")
                pass
            self.text.mark_set("insert", "end")
            self.currhist = self.lasthist
            self.text.see("insert")
            return "break"
        elif (key == "Up"):
            # Previous history
            if (self.cmd_in_progress):
                return "break"
            if (self.currhist == 0):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            if (self.currhist == self.lasthist):
                command = self.text.get(lastrow + ".2", lastrow + ".end")
                self.history[self.lasthist] = command
                pass
            self.text.delete(lastrow + ".2", lastrow + ".end")
            self.currhist -= 1
            self.text.insert(lastrow + ".2", self.history[self.currhist])
            return "break"
        elif (key == "Down"):
            if (self.cmd_in_progress):
                return "break"
            # Next history
            if (self.currhist == self.lasthist):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            self.text.delete(lastrow + ".2", lastrow + ".end")
            self.currhist += 1
            self.text.insert(lastrow + ".2", self.history[self.currhist])
            return "break"
        elif (len(event.char) == 1) and (event.char < chr(255)):
            # A key that will result in text addition.  Make sure it
            # only occurs on the last line and not in the prompt area.
            if (self.cmd_in_progress):
                return "break"
            (lastrow, lastcol) = self.text.index("end").split(".")
            lastrow = str(int(lastrow)-1)
            (currrow, currcol) = self.text.index("insert").split(".")
            if ((lastrow != currrow) or (int(currcol) < 2)):
                # Ignore the keypress
                return "break"
            pass
        elif ((key == "Left") or (key == "Right") or
              (key == "Insert") or
              (key == "End") or (key == "Home") or
              (key == "Prior") or (key == "Next")):
            # Pass these through
            return
        else:
            return "break"
        return

    pass

def cmphist(a, b):
    return cmp(a[0], b[0])
    
def _HistorySave(file):
    if (not init_history):
        return
    domimpl = xml.dom.getDOMImplementation()
    doc = domimpl.createDocument(None, "IPMIHistory", None)
    main = doc.documentElement
    i = 0
    for cmd in init_history:
        if (cmd != ""):
            helem = doc.createElement("hval")
            helem.setAttribute("idx", str(i))
            helem.setAttribute("val", cmd)
            main.appendChild(helem)
            i += 1
            pass
        pass
    try:
        info = os.stat(file)
        pass
    except:
        # File doesn't exist, create it.
        try:
            fd = os.open(file, os.O_WRONLY | os.O_CREAT,
                         stat.S_IRUSR | stat.S_IWUSR)
            os.close(fd)
            pass
        except:
            _oi_logging.error("Unable to create startup file " + file)
            return
        pass
    try:
        f = open(file, 'w')
        doc.writexml(f, indent='', addindent='\t', newl='\n')
    except:
        pass
    return

def _HistoryRestore(file):
    info = None
    try:
        info = os.stat(file)
    except:
        pass
    if (info):
        if ((info.st_mode & (stat.S_IRWXG | stat.S_IRWXO)) != 0):
            sys.exit("The history file '" + file + "' is group or world"
                     + " accessible.  It contains passwords, and"
                     + " should be secure.  Not starting the GUI,"
                     + " please fix the problem first.")
            return
        pass
    try:
        doc = xml.dom.minidom.parse(file).documentElement
    except:
        return
    for c in doc.childNodes:
        if (c.nodeType == c.ELEMENT_NODE):
            try:
                idx = int(c.getAttribute("idx"))
                val = c.getAttribute("val")
                init_history.append( (idx, val) )
                pass
            except:
                pass
            pass
        pass
    init_history.sort(cmphist)
    return



# _fru.py
#
# openipmi GUI handling for FRU data
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import _oi_logging
import gui_treelist
import gui_popup
import gui_setdialog

class ReinitOnAny:
    def __init__(self, glist):
        self.glist = glist;
        return

    def reinit(self, data, s_oldval, s_newval):
        oldval = int(s_oldval)
        newval = int(s_newval)
        if (oldval != newval):
            self.glist.refresh()
            return True
        return False

    pass

class ReinitOnZero:
    def __init__(self, glist):
        self.glist = glist;
        return

    def reinit(self, data, s_oldval, s_newval):
        oldval = int(s_oldval)
        newval = int(s_newval)
        if (oldval != newval) and ((oldval == 0) or (newval == 0)):
            self.glist.refresh()
            return True
        return False

    pass

class FRUData:
    def __init__(self, glist, node, index, pname, ptype, origval, parent,
                 settable, reiniter = None):
        self.glist = glist
        self.node = node
        self.aidx = index
        self.pname = pname
        self.ptype = ptype
        self.origval = origval
        self.currval = origval
        self.parent = parent
        if (parent != None):
            parent.children.insert(index, self)
            pass
        self.reiniter = reiniter
        self.settable = settable
        if (ptype == "binary") or (ptype == "unicode"):
            self.longtext = True
            pass
        else:
            self.longtext = False;
            pass

        enums = None
        cpos = [ -1 ]
        npos = [ 0 ]
        val = [ "" ]
        rv = self.node.get_enum_val(self.aidx, cpos, npos, val)
        if (rv == 0):
            enums = [ ]
            while (rv == 0):
                enums.append(val[0])
                if (npos[0] == -1):
                    break
                cpos[0] = npos[0]
                val = [ "" ]
                rv = self.node.get_enum_val(self.aidx, cpos, npos, val)
                pass
            pass
        self.enums = enums
        return

    def do_on_close(self):
        self.parent = None
        return
        
    def SetItem(self, item):
        self.item = item
        return

    def HandleMenu(self, event, idx, point):
        menul = [ ]
        if (self.settable):
            if (self.ptype == "boolean"):
                menul.append( ("Toggle Value", self.togglevalue) )
                pass
            else:
                menul.append( ("Set Value", self.setvalue) )
                if (self.enums != None):
                    for v in self.enums:
                        menul.append( (v, self.setenum, v) )
                        pass
                    pass
                pass
            pass
        if (self.parent != None):
            menul.append( ("Delete", self.delete_item) )
            menul.append( ("Insert Before", self.ins_before) )
            pass
        if (len(menul) > 0):
            gui_popup.popup(self.glist, event, menul, point)
            pass
        return

    def setenum(self, val):
        self.ok([ val ])
        return
    
    def ok(self, vals):
        rv = self.node.set_field(self.aidx, self.ptype, str(vals[0]))
        if (rv != 0):
            self.glist.SetError("Invalid data value: "
                                + OpenIPMI.get_error_string(rv))
            return
        try:
            if (self.reiniter != None):
                if self.reiniter.reinit(self, self.currval, vals[0]):
                    return
                pass
            pass
        except:
            pass
        
        name_s = [ "" ]
        type_s = [ "" ]
        value_s = [ "" ]
        node_s = [ None ]
        rv = self.node.get_field(self.aidx, name_s, type_s, value_s, node_s)
        if (rv != 0):
            self.glist.SetError("Could not re-get field: "
                                + OpenIPMI.get_error_string(rv))
            return

        self.glist.cleanup_field(type_s[0], value_s)
        self.currval = value_s[0]
        self.glist.SetColumn(self.item, self.currval, 1)
        return
    
    def delete_item(self, event):
        self.parent.delete_element(self)
        return
    
    def ins_before(self, event):
        self.parent.insert_element(self)
        return
    
    def setvalue(self, event):
        gui_setdialog.SetDialog("Set value for " + self.pname,
                                [ self.currval ], 1, self,
                                longtext=self.longtext)
        return

    def togglevalue(self, event):
        if (self.currval == "True"):
            newval = "False"
        else:
            newval = "True"
            pass
        rv = self.node.set_field(self.aidx, self.ptype, newval)
        if (rv != 0):
            self.glist.SetError("Could not toggle value: "
                                + OpenIPMI.get_error_string(rv))
            return
        self.currval = newval
        self.glist.SetColumn(self.item, newval, 1)
        return

    pass

class ArrayFRUData(FRUData):
    def __init__(self, glist, node, index, subnode, pname, ptype, length,
                 parent, settable):
        FRUData.__init__(self, glist, node, index, pname, ptype, length,
                         parent, settable, False)
        self.subnode = subnode
        self.children = [ ]
        self.length = length
        return

    def do_on_close(self):
        self.children = [ ]
        FRUData.do_on_close(self)
        return
        
    def HandleMenu(self, event, idx, point):
        menul = [ ]
        if (self.settable):
            menul.append( ("Add an array element", self.add_element) )
            pass
        if (self.parent != None):
            menul.append( ("Delete", self.delete_item) )
            menul.append( ("Insert Before", self.ins_before) )
            pass
        if (len(menul) > 0):
            gui_popup.popup(self.glist, event, menul, point)
        return

    def reidx_children(self):
        i = 0
        for l in self.children:
            l.aidx = i
            self.glist.SetColumn(l.item, str(i), 0)
            i += 1
            pass
        return

    def redisplay_item(self, idx, before=None):
        name_s = [ "" ]
        type_s = [ "" ]
        value_s = [ "" ]
        node_s = [ None ]
        rv = self.subnode.get_field(idx, name_s, type_s, value_s, node_s)
        if (rv != 0):
            self.glist.SetError("Could not get field: "
                                + OpenIPMI.get_error_string(rv))
            return
        self.glist.add_fru_data(self.item, self.subnode, idx,
                                self, False, idx, before)
        return
    
    def add_element(self, event):
        self.node.set_field(self.aidx, "integer", str(self.length))
        self.redisplay_item(self.length)
        self.length += 1
        return

    def delete_element(self, child):
        rv = self.node.set_field(self.aidx, "integer", str(-(child.aidx+1)))
        if (rv != 0):
            self.glist.SetError("Could not delete item: "
                                + OpenIPMI.get_error_string(rv))
            return
        self.glist.Remove(child.item)
        del self.children[child.aidx]
        self.reidx_children()
        self.length -= 1
        return
        
    def insert_element(self, child):
        rv = self.node.set_field(self.aidx, "integer", str(child.aidx))
        if (rv != 0):
            self.glist.SetError("Could not insert item: "
                                + OpenIPMI.get_error_string(rv))
            return
        self.redisplay_item(child.aidx, child.item)
        self.reidx_children()
        self.length += 1
        return

    pass

class FruInfoDisplay(gui_treelist.TreeList):
    def __init__(self, fru, name):
        self.fru = fru
        name_s = [ "" ]
        node_s = [ None ]
        rv = fru.get_root_node(name_s, node_s)
        if (rv != 0):
            _oi_logging.error("unable to get FRU node: " + str(rv))
            return

        self.fru_type = name_s[0]
        
        gui_treelist.TreeList.__init__(self, "FRU info for " + name,
                                       ".",
                                       [("Name", 300), ("Value", 300)])

        self.add_fru_data(self.treeroot, node_s[0], 0, None,
                          self.fru_type == "standard FRU")
        self.AfterDone()
        self.ExpandItem("")
        return

    def do_on_close(self):
        self.fru = None
        return
        
    def cancel(self):
        self.Close()
        return

    def fru_written(self, domain, fru, err):
        if (err):
            self.glist.SetError("Could not write FRU: "
                                + OpenIPMI.get_error_string(err))
            return
        self.Close()
        return
    
    def save(self):
        self.fru.write(self)
        self.Close()
        return

    def cleanup_field(self, type, value_s):
        if (type == "binary"):
            splitup = value_s[0].split()
            p = 0
            value_s[0] = ""
            if (len(splitup) > 0):
                value_s[0] += splitup[0]
                del splitup[0]
                pass
            for v in splitup:
                p += 1
                if (p >= 8):
                    value_s[0] += "\n"
                    p = 0
                    pass
                else:
                    value_s[0] += " "
                    pass
                value_s[0] += v
                pass
            pass
        return
    
    def refresh(self):
        name_s = [ "" ]
        node_s = [ None ]
        rv = self.fru.get_root_node(name_s, node_s)
        if (rv != 0):
            _oi_logging.error("unable to get FRU node: " + str(rv))
            return

        self.fru_type = name_s[0]
        self.RemoveAll()
        self.add_fru_data(self.treeroot, node_s[0], 0, None,
                          self.fru_type == "standard FRU")
        self.AfterDone()
        self.ExpandItem("")
        return
    
    def add_fru_data(self, item, node, startidx, parent, normal_top,
                     endidx=-1, before=None):
        i = startidx
        while True:
            name_s = [ "" ]
            type_s = [ "" ]
            value_s = [ "" ]
            node_s = [ None ]
            rv = node.get_field(i, name_s, type_s, value_s, node_s)
            if (rv == OpenIPMI.einval):
                return
            if (type_s[0] == "boolean"):
                value_s[0] = str(bool(int(value_s[0])))
            # Ignore other errors, just keep going
            if (rv == 0):
                if (name_s[0] == None):
                    name_s[0] = str(i)
                    pass
                if (type_s[0] == "subnode"):
                    np = None
                    if (node.settable(i) == 0) and (value_s[0] != "-1"):
                        # A settable array
                        data = ArrayFRUData(self, node, i, node_s[0],
                                            name_s[0], node_s[0].get_subtype(),
                                            int(value_s[0]), parent, True)
                        np = data
                        pass
                    else:
                        data = FRUData(self, node, i, name_s[0], type_s[0],
                                       None, parent, node.settable(i) == 0,
                                       False)
                        pass
                    sub = self.add_data(item, name_s[0], [], data,
                                        before=before)
                    self.add_fru_data(sub, node_s[0], 0, np, False)
                else:
                    reiniter = None
                    if (normal_top):
                        if (name_s[0] == "multi_record_offset"):
                            reiniter = ReinitOnAny(self)
                            pass
                        elif name_s[0].endswith("_offset"):
                            reiniter = ReinitOnZero(self)
                            pass
                        pass
                    self.cleanup_field(type_s[0], value_s)
                    data = FRUData(self, node, i, name_s[0], type_s[0],
                                   value_s[0], parent, node.settable(i) == 0,
                                   reiniter)
                    self.add_data(item, name_s[0], [value_s[0]], data=data,
                                  before=before)
                    pass
                pass
            if (endidx >= 0) and (i == endidx):
                break
            i = i + 1
            pass
        return
    
    pass
